%********************    PROTEIN V2.32   ********************

%********************     t_header.pl    ********************

% Praedikate zur Durchfuehrung bzw Start des eigentlichen Beweises.

%************************************************************

:- module_interface( protein_proof).

:- global proof / 0,
	  protein_result / 1.

:- export protein_ae / 2.

:- begin_module( protein_proof).

% Unterdrueckung von "indefinite delay" Meldungen
:- set_error_handler( 272, true/0).

:- dynamic protein_result / 1,
	   protein_ae / 2.

protein_mode( proof).


% ---------- Haupteinsprungpunkt ----------

%%% proof
%%% Haupteinsprungpunkt zum Start des eigentlichen Beweises

proof :-
	h_init,
	h_proof( Trace),
	h_output( Trace), !.


%%% h_init
%%% Initialisierungen, Ausgabe Hinweise an den Benutzer

h_init :- 
	def_const( problem, Problem),
	(protein_flag( calculus, me) ; protein_flag( delayed_rme, off) -> true;
            setval( block, 0)),
	protein_flag( mmr, MMR),
	protein_flag( ec_pruning, ECP), 
	((MMR == gcwa ; MMR == model ; ECP == on) -> 
	    setval( restart, 0),
	    ((protein_flag(ctest,protein) ; MMR == model ; ECP == on) -> true;
		compile( "$HYPERHOME/hyper", t_hyper),
		use_module( t_hyper),
		protein_cc_init( Problem))
        ; true),
	(protein_flag( search, id_inf) -> setval( id_inf, 0); true),
	h_init_stat,
	h_init_dynamic,
	u_out( "%nproving %w...%n", [Problem]).

 
%%% h_proof( Trace)
%%% Start des Beweises mit Statistiken

h_proof( Trace) :-
	protein_flag( search, Search),
	(Search == id_tree -> u_out( "tree depth ") ;
        (Search == id_inf  -> u_out( "inference maximum ") ;
	(Search == id_term -> u_out( "term depth "); true))),
        statistics( runtime, _),
	h_retract,
        (h_start_proof( DepthOut, Trace) ; true),
        statistics( runtime, [_,Time]),
	Seconds is Time * 0.001,
	setval( proof_time, Seconds),
	(protein_flag( search, prolog) -> true; setval( proof_depth, DepthOut)).


%%% h_retract
%%% Loeschen aller dynamischen EIntraege fuer die neue Runde

h_retract :-
 	retract_all( protein_result(_)),
	retract_all( protein_ae(_,_)).


%%% h_output( Trace)
%%% Ausgabe der Statistik, Antworten, Trace

:- mode h_output( +).

h_output( Trace) :-    
	protein_result( proved), !,
	protein_flag( out_stream, S),
	h_write_stat( S), 
        h_write_answers( S),
        h_write_more_answers,
	h_write_tree( Trace),
        h_write_trc.

h_output( _) :-
	protein_result( indefinite), !,
	protein_flag( out_stream, S),
	h_write_stat( S), 
	u_out( "--- indefinite ---%n").

h_output( _) :-
	protein_flag( mmr, MMR),
	(MMR == gcwa ; MMR == model),
	protein_flag( ctest, CTest),
	(CTest == end ; CTest == protein ; MMR == model), !,
	protein_flag( out_stream, S),
	h_write_stat( S),
	h_mmr.

h_output( _) :-
	protein_flag( mmr, gcwa), !,
	protein_flag( out_stream, S),
	h_write_stat( S),
        u_out( "--- negation by failure to explain ---%n"). 
       
h_output( _) :-
	protein_flag( out_stream, S),
	h_write_stat( S),
        u_out( "--- failure ---%n"). 
       
        
% ---------- iterative deepening ----------

%%% h_search( Goal, Tiefe in Query, Eingabetiefe, Inkrement, Ausgabetiefe) 
%%% Suchen des Beweisen mit iterative deepening
%%% Die globale Variable more_depth wird benutzt, um zu ermitteln, ob eine
%%% Erhoehung der Tiefe noch sinnvoll ist. 

:- mode h_search( +, ?, +, ++, -).

% Starten des Beweises mit Tiefe Depth
h_search( Goal, Depth, Depth, _, Depth) :-
	h_print_depth( Depth),
        Goal,  
	(protein_result( _) -> true ;
           u_out( "+++ proved +++%n"),
           assert( protein_result( proved))).

% Erhoehen der Tiefe um Inc und Restart des Beweises
h_search( Goal, QDepth, InDepth, Inc, OutDepth) :-
        getval( more_depth, More_depth),
        (More_depth == 0 ->
           InDepth = OutDepth,
	   u_out( "--- proof not possible ---%n"), !
	;
           ZwiDepth is InDepth + Inc,
	   h_retract,
	   h_search( Goal, QDepth, ZwiDepth, Inc, OutDepth)).


%%% h_print_depth( Depth)
%%% Schreibt aktuelle Tiefe und initialisiert neues Level

:- mode h_print_depth( ++).

h_print_depth( Depth) :-
	u_out( "%w,", [Depth]),
	setval( more_depth, 0).

  
%---------- Antworten ----------

%%% h_write_answers( S)
%%% Ausgabe gefundene Antworten

h_write_answers( S) :-
	delayed_goals( DL),
        (protein_flag( answer_set_handling, on) ->
	   setof( Answer, member( protein_answer( Answer), DL), AnswerL)
        ;
	   bagof( Answer, member( protein_answer( Answer), DL), AnswerL)),
	printf( S, "%nAnswers%n-------%n", []),
	h_do_write_answers( S, AnswerL), !.
       
h_write_answers( _) :- !.
        

%%% h_do_write_answers( S, AnswerL)
%%% Schreibt gefundene Antwort

:- mode h_do_write_answers( ++, +).

h_do_write_answers( S, [Answer]) :- writeln( S, Answer), !.

h_do_write_answers( S, [Answer|AnswerR]) :-
	write( S, Answer), 
        write( S, ' ; '),
	h_do_write_answers( S, AnswerR).


%%% h_write_more_answers
%%% stoesst gegebenenfalls das Suchen nach weiteren Beweisen an.

h_write_more_answers :-
        protein_flag( answers, more),
	protein_flag( search, Search),
	u_out( "%n   More? (;)"),
	u_on_backtrack( h_retract), 
	(Search == id_tree -> u_on_backtrack( u_out("tree_depth "));
        (Search == id_inf  -> u_on_backtrack( u_out("inference maximum "));
	(Search == id_term -> u_on_backtrack( u_out("term depth ")); true))),
        u_on_backtrack( u_out( ";%n")),
        \+ tyi( 59), !,
        u_out( "%n").  

h_write_more_answers :-
        protein_flag( answers, all),
	protein_flag( search, Search),
	u_out( "%n"),	
	u_on_backtrack( h_retract), 
	(Search == id_tree -> u_on_backtrack( u_out("tree_depth "));
        (Search == id_inf  -> u_on_backtrack( u_out("inference maximum "));
	(Search == id_term -> u_on_backtrack( u_out("term depth ")); true))),
        u_on_backtrack( u_out( "%n")), 
        fail, !.

h_write_more_answers :- protein_flag( answers, one), !.

% END t_header.pl END

h_mmr.

%********************  t_header_info.pl  ********************

% Praedikate fuer Beweisstatistiken

%************************************************************

%%% h_write_trans_stat( S)
%%% Ausgabe der Uebersetzungsstatistik

:- mode h_write_trans_stat( ++).

h_write_trans_stat( S) :-
	nl( S),
	(protein_flag( reorder, off) -> true;
           getval( reorder, Reorder),
	   printf( S, "   Static Reorderings:    %5d%n", [Reorder])),
        (def_const( simplify, yes) -> 
	   getval( simplify, Simplify),
	   getval( sim_time, SimTime),
	   printf( S, "   Static Simplifications:%5d%n", [Simplify]),
           FSimTime is SimTime * 1.0,
           printf( S, "   Simplification Time:     %5.1f sec%n", [FSimTime])
        ; true),
	getval( taut, Taut),
        (Taut > 0 ->
           printf( S, "   Tautology Deletions:   %5d%n", [Taut]); true), !.


%%% p_count( GlobaleVariablenname)
%%% Bei jedem Schritt des Beweises wird entsprechend des Typs des Schritts
%%% eine globale Variable erhoeht. (Backtrackingfaehig)

:- mode p_count( ++).
	
p_count( Type) :- incval( Type), u_on_backtrack( decval( Type)). 


%%% h_init_stat
%%% Initialisierung der globalen Variablen fuer die Statistik 

h_init_stat :-
	setval( ext, 0),
	setval( red, 0),
	setval( red_cut, 0),
	setval( restart, 0),
	setval( factor, 0),
	setval( factor_cut, 0),
	setval( theory, 0),
        setval( inf, 0), 
	setval( proof_depth, 0),
	setval( proof_time, 0.0),
	setval( sim, 0),
	setval( sim_fail, 0),
	setval( sim_time, 0).


%%% h_write_stat( S)
%%% Schreibt Statistiken

:- mode h_write_stat( ++).

h_write_stat( S)  :-
	getval( ext, Ext),
	getval( red, Red_Nocut),
	getval( red_cut, Red_Cut),
	getval( restart, Restart),
	getval( factor, Factor_Nocut),
	getval( factor_cut, Factor_Cut),
	getval( sim, Sim),
	getval( sim_fail, SimFail),
	getval( theory, Theory),
	getval( inf, Inf),
        getval( proof_time, Proof_Time),
        (protein_flag( search, id_tree) -> 
	    getval( proof_depth, Proof_Depth); true),
	Red is Red_Nocut + Red_Cut,
	Factor is Factor_Nocut + Factor_Cut,
	nl( S),
	    printf( S, "   Proof Time:              %5.1f sec%n", [Proof_Time]),
        (protein_flag( search, id_tree) ->
            printf( S, "   Proof Depth:           %5d%n", [Proof_Depth]); true),
	(protein_result( proved) ->
            printf( S, "   Extension Steps:       %5d%n", [Ext]),
           (protein_flag( calculus, rme) ->
            printf( S, "   Restart Steps:         %5d%n", [Restart]); true),
            printf( S, "   Reduction Steps:       %5d", [Red]), 
           ((protein_flag( reduction, cut) ; 
             protein_flag( reduction, both)) ->
	    printf( S, " (incl.%6d with cut)%n", [Red_Cut]); nl(S)),
	    printf( S, "   Factoring Steps:       %5d", [Factor]),
           ((protein_flag( factorisation, cut) ; 
             protein_flag( factorisation, both)) ->
	    printf( S, " (incl.%6d with cut)%n", [Factor_Cut]); nl(S)),
	   (def_const( simplify, yes) ->
	    printf( S, "   Simplification Steps:  %5d", [Sim]),
           (SimFail > 0 ->
            printf( S, " (add.%7d forced failures)%n", [SimFail]); nl(S));true),
	   (def_const( theory, yes) ->
	    printf( S, "   Theory Steps:          %5d%n", [Theory]);true)
        ; true),
	    printf( S, "   Inferences:          %7d%n", [Inf]).


% ---------- Beweisinformationen schreiben ----------

%%% h_write_trc
%%% Schreibt die Beweisinfos in die Datei [Input-File-Name].trc

h_write_trc :-
	u_open_trc( S),
	printf( S,
        "%n------------------------ statistics ------------------------%n", []),
        h_write_trans_stat( S),
	h_write_stat( S),
	printf( S, 
       "%n------------------------- answers --------------------------%n", []),
        h_write_answers( S),
        close( S).

% END t_header_info.pl END

h_write_tree(_10057).

h_init_dynamic.

%********************  t_header_utils.pl ********************

% Utility-Praedikate, die waehrend des Beweisens gebraucht werden. 

%************************************************************

%%% p_unifiable( Element, List)
%%% Element unifizierbar mit einem Element der Liste? 

:- mode p_unifiable( +, +).

p_unifiable( El, [El|_]).
p_unifiable( El, [_,El|_]).
p_unifiable( El, [_,_,El|_]).
p_unifiable( El, [_,_,_|R]) :- p_unifiable( El, R).


%%% p_unifiable_n_count( Element, List, Index)
%%% Element unifizierbat mit dem wievielten Element der Liste?

:- mode p_unifiable_n_count( +, +, ?).

p_unifiable_n_count( El, [El|_], 1).
p_unifiable_n_count( El, [_,El|_], 2).
p_unifiable_n_count( El, [_,_,El|_], 3).

p_unifiable_n_count( El, [_,_,_|R], N) :-
	p_unifiable_n_count( El, R, M),
	N is M + 3.


%%% p_identical( Element, List)
%%% Element identisch eines Elements der Liste?

:- mode p_identical( +, +).

p_identical( El, [X|_])     :- El == X, !.
p_identical( El, [_,X|_])   :- El == X, !.
p_identical( El, [_,_,X|_]) :- El == X, !.
p_identical( El, [_,_,_|R]) :- p_identical( El, R), !.


%%% p_identical_n_count( Element, Liste, Index)
%%% Element identisch des wievielten Elements der Liste?

:- mode p_identical_n_count( +, +, -).

p_identical_n_count( El, [X|_], 1)     :- El == X, !.
p_identical_n_count( El, [_,X|_], 2)   :- El == X, !.
p_identical_n_count( El, [_,_,X|_], 3) :- El == X, !.

p_identical_n_count( El, [_,_,_|R], N) :- 
	p_identical_n_count( El, R, M), 
	N is M + 3, !.


%%% p_identical_upto_restart( Element, List)
%%% Element identisch eines Elements der Liste bis zum letzten Restart?

:- mode p_identical_upto_restart( +, +).

p_identical_upto_restart( _, [restart|_])       :- !, fail.
p_identical_upto_restart( El, [X|_])            :- El == X, !.
p_identical_upto_restart( _, [_,restart|_])     :- !, fail.
p_identical_upto_restart( El, [_,X|_])          :- El == X, !.
p_identical_upto_restart( _, [_,_,restart|_])   :- !, fail.
p_identical_upto_restart( El, [_,_,X|_])        :- El == X, !. 
p_identical_upto_restart( El, [_,_,_|R]) :- p_identical_upto_restart( El, R), !.


%%% p_never_identical( Element, Liste)
%%% Element nie identisch eines Elements der Liste?

:- mode p_never_identical( +, +).

p_never_identical( _, []).

p_never_identical( El, [X|R]) :-
	!, El ~= X,
        p_never_identical( El, R).


%%% p_never_identical_upto_restart( Element, AncL)
%%% Elememt nie identisch eines Elements der Liste bis zum letzten Restart?

:- mode p_never_identical_upto_restart( +, +).

p_never_identical_upto_restart( _, []).
p_never_identical_upto_restart( _, [restart|_]) :- !.

p_never_identical_upto_restart( El, [X|R]) :-
	!, El ~= X,
        p_never_identical_upto_restart( El, R).


%%% u_rewrite_list( Flag, SimLit, RewriteL, InCondL, InLitL, OutLitL, OutCondL)
%%% Wendet p_rewrite auf jedes Literal der InLitL an

:- mode u_rewrite_list( ++, +, +, +, ?, -, -).

u_rewrite_list( _, _, _, _, [], [], []) :- !.

u_rewrite_list( Flag, SimTerm, RewriteL, InCondL, [InTerm|InTermR], 
           [OutTerm|OutTermR], [OutCondL|OutCondR]) :- !,
        p_rewrite( Flag, SimTerm, RewriteL, InCondL, InTerm, 
                   OutTerm, OutCondL),
        u_rewrite_list( Flag, SimTerm, RewriteL, InCondL, InTermR, 
                   OutTermR, OutCondR).		   


%%% p_rewrite( Flag, SimLit, RewriteL, InCondL, InLit, OutLitL, OutCondL)
%%% Rewrite matchender Literale durch RewriteL

:- mode p_rewrite( ++, +, +, +, ?, ?, -).

p_rewrite( _, _, _, _, Var, [Var], []) :- var( Var), !.

p_rewrite( con, SimLit, RewriteL, InCondL, InLit, [OutLit], OutCondL) :-
	instance( InLit, SimLit), 
	copy_term( (SimLit,RewriteL,InCondL), (CSimLit,CRewriteL,OutCondL)),
	CSimLit = InLit, !,
	member( OutLit, CRewriteL).
	
p_rewrite( dis, SimLit, RewriteL, InCondL, InLit, CRewriteL, OutCondL) :- 
	instance( InLit, SimLit), 
	copy_term( (SimLit,RewriteL,InCondL), (CSimLit,CRewriteL,OutCondL)),
	CSimLit = InLit, !.

p_rewrite( rew, SimTerm, RewriteL, InCondL, InTerm, OutTermL, OutCondL) :- 
	InTerm =.. [Fnc|InArgL],
	u_rewrite_list( rew, SimTerm, RewriteL,InCondL,InArgL,ZwiArgL,ZwiCondL),
	u_flatten( ZwiArgL, OutArgL),
	ZwiTerm =.. [Fnc|OutArgL],
	p_rewrite( dis, SimTerm, RewriteL, InCondL, ZwiTerm, OutTermL,OutCondH),
	flatten( [OutCondH|ZwiCondL], OutCondL), !.

p_rewrite( flat, SimTerm, [Rewrite], InCondL, InTerm, [OutTerm], OutCondL) :- 
	InTerm =.. [Fnc|InArgL],
	l_rewrite_some( SimTerm, Rewrite,InCondL,InArgL,OutArgL,OutCondL),
	OutTerm =.. [Fnc|OutArgL],
	!.

p_rewrite( _, _, _, _, X, [X], []) :- !.


l_rewrite_some( SimTerm, Rewrite, InCondL, 
	        [Term|Rest], [CRewrite|Rest], OutCondL) :-
	instance( Term, SimTerm), 
	copy_term( (SimTerm,Rewrite,InCondL), (CSimTerm,CRewrite,OutCondL)),
	CSimTerm = Term, 
%	printf('Rewrite %w to %w, cond %w%n',[Term, CRewrite, OutCondL]),
	!.

l_rewrite_some( SimTerm, RewriteL, InCondL, 
	        [Term|Rest], [Term|SimRest], OutCondL) :-
	l_rewrite_some( SimTerm, RewriteL, InCondL, Rest, SimRest, OutCondL).


%%% p_assert_bck( Term)
%%% Backtrackingfaehiges append

:- mode p_assert_bck( +).

p_assert_bck( Term) :- assert( Term), u_on_backtrack( retract(Term)).


%%% p_count( GlobaleVariablenname, Additor)
%%% Backtrackingfaehiger Zaehler

:- mode p_count( ++, ++).
	
p_count( Type, Add) :- 
	getval( Type, Value),
	plus( Value, Add, Sum),
	setval( Type, Sum),
	u_on_backtrack( setval( Type, Value)). 


%%% p_call( Goal)
%%% Prologaufrufe, die nicht im Trace erscheinen

:- mode p_call( +).

p_call( Goal) :- Goal.


%%% u_on_backtrack( Goal)
%%% Ausfuehrung des Goals beim Backtracking

:- mode u_on_backtrack( +).

u_on_backtrack( _).
u_on_backtrack( Goal) :- Goal, !, fail. 


%%% u_flatten( InLL, OutL)
%%% Wie Flatten, jedoch nur ein Level

:- mode u_flatten( +, -).

u_flatten( [], []).

u_flatten( [H|InR], L) :-
	u_flatten( InR, OutR), 
	append( H, OutR, L).


%%% u_delay( Variable, Goal)
%%% Delayen des Goals bis Variable instanziiert wurde
%%% pragma( debug). erforderlich (to be able to see the DELAY port)

u_delay(Term, Goal, Module) :-
	make_suspension(Goal, 2, Susp, Module),
	insert_suspension(Term, Susp, 1, suspend).


%%% u_open_trc( S)
%%% Oeffnen des Tracefiles

:- mode u_open_trc( -).

u_open_trc( S) :-
	def_const( problem, Problem),
	concat_atoms( Problem, '.trc', FileN),
	open( FileN, append, S).


% ---------- Bildschirmmeldungen ----------

%%% u_out( String)
%%% u_out( String, VarL)
%%% Gibt Message auf dem Bildschirm aus

:- mode u_out( ++).
:- mode u_out( ++, +).

u_out( String) :- u_out( String, []).

u_out( String, VarL) :-
	protein_flag( out_stream, S),
	printf( S, String, VarL),
	flush( S).
	

% ---------- Start und Ende ----------

%%% u_init( Problem, OutProblem)
%%% Initialisierungen, Ausgabe Hinweise an den Benutzer

%:- mode u_init( ++, -).

u_init( InProblem, Problem) :-  
	name( InProblem, InProblemL),
	(append( ProblemL, [46,116,109,101], InProblemL) -> true ;
	    ProblemL = InProblemL),
	name( Problem, ProblemL),  
        set_error_handler(139,true/0),
	statistics( runtime, [Start,_]),
	def_set_const( start, Start),
	def_set_const( problem, Problem),
	set_flag( occur_check, on), 
	set_flag( print_depth, 1000000).

	
%%% u_tini
%%% Zuruecksetzen der Initialisierungen

u_tini :- 
	((protein_flag( trace, dynamic(_)), protein_mode( proof)) ->
           close( tview), 
	   close( input),
	   delete( in),
	   delete( out); true),
        (def_const( header, HeaderF),
	 exists( HeaderF) ->
	   concat_atoms( 'rm ', HeaderF, Call),
           sh( Call); true),
	def_const( problem, Problem),
	def_const( start, Start),
	statistics( runtime, [End,_]),
	Time is End - Start,
	Seconds is Time * 0.001,
	u_out( "%n%w.tme   finished in %w seconds%n", [Problem,Seconds]), !.
    

%%% u_abort
%%% Abort mit Zuruecksetzen der Initialisierungen

u_abort :- 
	u_out( "%naborting...", []),
	u_tini, 
	(protein_flag( out_stream, user) ->
	    halt ; exit_block( protein)).

% END t_header_utils.pl END

% ---------- Prolog Text ----------

p_neg_shortlit(pos, query, not_query).
p_neg_shortlit(neg, not_query, query).
p_neg_shortlit(pos, restart, not_restart).
p_neg_shortlit(neg, not_restart, restart).
p_neg_shortlit(pos, theory(A, B, C), not_theory(A, B, C)).
p_neg_shortlit(neg, not_theory(A, B, C), theory(A, B, C)).
p_neg_shortlit(pos, none, not_none).
p_neg_shortlit(neg, not_none, none).
p_neg_shortlit(pos, protein_prolog(A), protein_prolog(A)).
p_neg_shortlit(pos, protein_zarg(A, B), not_protein_zarg(A, B)).
p_neg_shortlit(neg, not_protein_zarg(A, B), protein_zarg(A, B)).
p_neg_shortlit(neg, 'not_='(A, B), A = B).
p_neg_shortlit(pos, A = B, 'not_='(A, B)).
p_neg_shortlit(pos, A =< B, 'not_=<'(A, B)).
p_neg_shortlit(neg, 'not_=<'(A, B), A =< B).


:- set_flag( variable_names, on).


% ---------- Control Predicates ----------

%%% h_start_proof( OutDepth, Trace)
%%% Starten des Beweises mit iterative deepening,
%%% dabei Ermitteln der Beweistiefe.

h_start_proof(A, B) :-
	h_search(query(C, D, ([], []), ([], []), '_*'), D, 1, 1, A).


%%% protein_answer( Answer)
%%% Speicherung der Antwortendurch Verzoegerung dieses Praedikates.

protein_answer(A) :-
	u_delay(true, protein_answer(A)).


% ---------- Additional Arguments ----------

p_zarg(reg, 4).
p_zarg(depth, 3).
p_zarg(ancs, 2).
p_zarg(aunts, 1).
p_zarg(check, 0).


% ---------- Constants ----------

def_const(problem, 'triangle-eqax-th').
def_const(titel, "********************    PROTEIN V2.32   ********************").
def_const(theory, yes).
def_const(simplify, no).


% ---------- Flags ----------

protein_flag(ancestry_rme, off).
protein_flag(ancs, both).
protein_flag(answers, one).
protein_flag(answer_set_handling, on).
protein_flag(calculus, me).
protein_flag(check_flags, []).
protein_flag(ctest, now).
protein_flag(definite_answers, off).
protein_flag(delayed_rme, off).
protein_flag(depth_increment, 1).
protein_flag(dynamic, off).
protein_flag(ec_pruning, off).
protein_flag(factorisation, cut).
protein_flag(rewrite, flat).
protein_flag(head_behind_prolog, off).
protein_flag(max_rew_cond, 0).
protein_flag(mmr, off).
protein_flag(mode, pl1).
protein_flag(out_stream, user).
protein_flag(query_reuse, on).
protein_flag(reduction, both).
protein_flag(regularity, delay).
protein_flag(reorder, off).
protein_flag(search, id_tree).
protein_flag(selection_function, off).
protein_flag(sim_dynamic, nocut).
protein_flag(sim_static, uncond).
protein_flag(sim_deletion, complete).
protein_flag(sim_focus, query).
protein_flag(sorting, costs).
protein_flag(strict_rme, off).
protein_flag(th_nonewclauses, off).
protein_flag(th_reduction, both).
protein_flag(th_regularity, delay).
protein_flag(th_sidelit_anc, off).
protein_flag(timeout, 0).
protein_flag(trace, info).
protein_flag(translate, all).
protein_flag(count, off).


% ---------- Predicates ----------

query(n, A, (B, C), D, '_*') :-
	(
	    A >= 1
	->
	    plus(E, 1, A)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	=<(abs(x + y), abs(x) + abs(y), r, E, ([query|B], C), D, '_*').
theory(y, [0 =< x + y|A], B, C, D, E, F, '_*') :-
	theory(y, A, B, G, D, E, F, '_*').
theory(y, ['not_=<'(x + y, 0)|A], B, C, D, E, F, '_*') :-
	theory(y, A, B, G, D, E, F, '_*').
theory(y, [A + B = B + A|C], D, E, F, G, H, '_*') :-
	theory(y, C, D, I, F, G, H, '_*').
theory(y, [A + 0 = A|B], C, D, E, F, G, '_*') :-
	theory(y, B, C, H, E, F, G, '_*').
theory(y, [A + i(A) = 0|B], C, D, E, F, G, '_*') :-
	theory(y, B, C, H, E, F, G, '_*').
theory(y, ['not_=<'(abs(x + y), abs(x) + abs(y))|A], B, C, D, E, F, '_*') :-
	theory(y, A, B, G, D, E, F, '_*').
theory(y, [protein_prolog(A)|B], C, D, E, F, G, '_*') :-
	protein_prolog(A, H, I, F, J, '_*'),
	theory(y, B, C, K, E, F, L, '_*').
theory(A, [B|C], D, E, F, (G, H), I, '_*') :-
	p_neg_shortlit(J, B, K),
	p_call(J == pos -> p_identical(K, H) ; p_identical(K, G)),
	!,
	theory(A, C, D, L, F, (G, H), M, '_*').
theory(A, [B|C], D, E, F, (G, H), I, '_*') :-
	p_neg_shortlit(J, B, K),
	p_call(J == pos -> p_unifiable(K, H) ; p_unifiable(K, G)),
	theory(A, C, D, L, F, (G, H), M, '_*').
theory(A, [], B, C, D, B, E, '_*').
theory(y, [A =< B|C], D, E, F, G, H, '_*') :-
	(
	    F >= 2
	->
	    plus(I, 2, F)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, C, D, J, F, G, H, '_*'),
	'not_=<'(B, A, K, I, G, H, '_*').
theory(y, [A =< B|C], D, E, F, G, H, '_*') :-
	(
	    F >= 2
	->
	    plus(I, 2, F)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, C, D, J, F, G, H, '_*'),
	'not_=<'(B, A, K, I, G, H, '_*').
theory(y, [A + B =< C + B|D], E, F, G, H, I, '_*') :-
	(
	    G >= 2
	->
	    plus(J, 2, G)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, D, E, K, G, H, I, '_*'),
	=<(A, C, L, J, H, I, '_*').
theory(y, ['not_=<'(A, B)|C], D, E, F, G, H, '_*') :-
	(
	    F >= 2
	->
	    plus(I, 2, F)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, C, D, J, F, G, H, '_*'),
	'not_=<'(A + K, B + K, L, I, G, H, '_*').
theory(y, [abs(A) = A|B], C, D, E, F, G, '_*') :-
	(
	    E >= 2
	->
	    plus(H, 2, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, B, C, I, E, F, G, '_*'),
	=<(0, A, J, H, F, G, '_*').
theory(y, ['not_=<'(0, A)|B], C, D, E, F, G, '_*') :-
	(
	    E >= 2
	->
	    plus(H, 2, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, B, C, I, E, F, G, '_*'),
	'not_='(abs(A), A, J, H, F, G, '_*').
theory(y, [abs(A) = i(A)|B], C, D, E, F, G, '_*') :-
	(
	    E >= 2
	->
	    plus(H, 2, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, B, C, I, E, F, G, '_*'),
	=<(A, 0, J, H, F, G, '_*').
theory(y, ['not_=<'(A, 0)|B], C, D, E, F, G, '_*') :-
	(
	    E >= 2
	->
	    plus(H, 2, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	theory(y, B, C, I, E, F, G, '_*'),
	'not_='(abs(A), i(A), J, H, F, G, '_*').
protein_prolog(A, B, C, D, E, '_*') :-
	A.
protein_zarg(reg, A, A, B, C, D, '_*').
protein_zarg(depth, A, B, A, C, D, '_*').
protein_zarg(ancs, A, B, C, A, D, '_*').
protein_zarg(aunts, A, B, C, D, A, '_*').
protein_zarg(check, '_*', A, B, C, D, '_*').
'not_='(A, B, r, C, (D, E), F, '_*') :-
	!,
	p_never_identical('not_='(A, B), E),
	'not_='(A, B, n, C, (D, E), F, '_*').
'not_='(A, B, n, C, (D, E), F, '_*') :-
	p_identical(A = B, D),
	!,
	incval(inf),
	p_count(red_cut).
'not_='(A, B, n, C, D, (E, F), '_*') :-
	p_identical('not_='(A, B), F),
	!,
	incval(inf),
	p_count(factor_cut).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	p_unifiable(A = B, D),
	incval(inf),
	p_count(red).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(B, A), E),
	theory(y, [], H, I, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(B, A, J, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + H, B + H), E),
	theory(y, [], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(A + H, B + H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H + A, H + B), E),
	theory(y, [], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H + A, H + B, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(A), i(B)), E),
	theory(y, [], H, I, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(i(A), i(B), J, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(A), abs(B)), E),
	theory(y, [], H, I, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(abs(A), abs(B), J, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A + B + C, D, n, E, (F, G), H, '_*') :-
	(
	    E >= 1
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + (B + C), D), G),
	theory(y, [], J, K, E, (F, ['not_='(A + B + C, D)|G]), H, '_*'),
	'not_='(A + (B + C), D, L, I, (F, ['not_='(A + B + C, D)|G]), H, '_*'),
	p_count(theory).
'not_='(A, B + (C + D), n, E, (F, G), H, '_*') :-
	(
	    E >= 1
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, B + C + D), G),
	theory(y, [], J, K, E, (F, ['not_='(A, B + (C + D))|G]), H, '_*'),
	'not_='(A, B + C + D, L, I, (F, ['not_='(A, B + (C + D))|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, H), E),
	theory(y, [B = H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(A, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, B), E),
	theory(y, [H = A], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, B, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = B, D),
	theory(y, ['not_='(H, A)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, B, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = A, D),
	theory(y, ['not_='(B, H)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, A, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(B, H), E),
	theory(y, [A =< H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_=<'(B, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(H, B), E),
	theory(y, [H =< A], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_=<'(H, B, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(A, H), E),
	theory(y, [B =< H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_=<'(A, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(H, A), E),
	theory(y, [H =< B], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_=<'(H, A, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, ['not_=<'(B, H)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(A, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, ['not_=<'(H, B)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, A, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< H, D),
	theory(y, ['not_=<'(A, H)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(B, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, ['not_=<'(H, A)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, B, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + H, I), E),
	theory(y, [B + H = I], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(A + H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A + B, C, n, D, (E, F), G, '_*') :-
	(
	    D >= 2
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(I + B, C), F),
	theory(y, [I = A], J, K, D, (E, ['not_='(A + B, C)|F]), G, '_*'),
	'not_='(I + B, C, L, H, (E, ['not_='(A + B, C)|F]), G, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, B + I), E),
	theory(y, [H = A + I], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, B + I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B + C, n, D, (E, F), G, '_*') :-
	(
	    D >= 2
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, I + C), F),
	theory(y, [B = I], J, K, D, (E, ['not_='(A, B + C)|F]), G, '_*'),
	'not_='(A, I + C, L, H, (E, ['not_='(A, B + C)|F]), G, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H + A, I), E),
	theory(y, [H + B = I], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H + A, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A + B, C, n, D, (E, F), G, '_*') :-
	(
	    D >= 2
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + I, C), F),
	theory(y, [I = B], J, K, D, (E, ['not_='(A + B, C)|F]), G, '_*'),
	'not_='(A + I, C, L, H, (E, ['not_='(A + B, C)|F]), G, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, I + B), E),
	theory(y, [H = I + A], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, I + B, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B + C, n, D, (E, F), G, '_*') :-
	(
	    D >= 2
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, B + I), F),
	theory(y, [C = I], J, K, D, (E, ['not_='(A, B + C)|F]), G, '_*'),
	'not_='(A, B + I, L, H, (E, ['not_='(A, B + C)|F]), G, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + H + I, J), E),
	theory(y, [B + H + I = J], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(A + H + I, J, M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A + B + C, D, n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(J + B + C, D), G),
	theory(y, [J = A], K, L, E, (F, ['not_='(A + B + C, D)|G]), H, '_*'),
	'not_='(J + B + C, D, M, I, (F, ['not_='(A + B + C, D)|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, B + I + J), E),
	theory(y, [H = A + I + J], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, B + I + J, M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B + C + D, n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, J + C + D), G),
	theory(y, [B = J], K, L, E, (F, ['not_='(A, B + C + D)|G]), H, '_*'),
	'not_='(A, J + C + D, M, I, (F, ['not_='(A, B + C + D)|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H + (A + I), J), E),
	theory(y, [H + (B + I) = J], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H + (A + I), J, M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A + (B + C), D, n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + (J + C), D), G),
	theory(y, [J = B], K, L, E, (F, ['not_='(A + (B + C), D)|G]), H, '_*'),
	'not_='(A + (J + C), D, M, I, (F, ['not_='(A + (B + C), D)|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, I + (B + J)), E),
	theory(y, [H = I + (A + J)], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, I + (B + J), M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B + (C + D), n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, B + (J + D)), G),
	theory(y, [C = J], K, L, E, (F, ['not_='(A, B + (C + D))|G]), H, '_*'),
	'not_='(A, B + (J + D), M, I, (F, ['not_='(A, B + (C + D))|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = A + I, D),
	theory(y, ['not_='(H, B + I)], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, A + I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H + A + I, J), E),
	theory(y, [H + B + I = J], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H + A + I, J, M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A + B + C, D, n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + J + C, D), G),
	theory(y, [J = B], K, L, E, (F, ['not_='(A + B + C, D)|G]), H, '_*'),
	'not_='(A + J + C, D, M, I, (F, ['not_='(A + B + C, D)|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, I + B + J), E),
	theory(y, [H = I + A + J], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, I + B + J, M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B + C + D, n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, B + J + D), G),
	theory(y, [C = J], K, L, E, (F, ['not_='(A, B + C + D)|G]), H, '_*'),
	'not_='(A, B + J + D, M, I, (F, ['not_='(A, B + C + D)|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H + (I + A), J), E),
	theory(y, [H + (I + B) = J], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H + (I + A), J, M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A + (B + C), D, n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A + (B + J), D), G),
	theory(y, [J = C], K, L, E, (F, ['not_='(A + (B + C), D)|G]), H, '_*'),
	'not_='(A + (B + J), D, M, I, (F, ['not_='(A + (B + C), D)|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, I + (J + B)), E),
	theory(y, [H = I + (J + A)], K, L, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, I + (J + B), M, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B + (C + D), n, E, (F, G), H, '_*') :-
	(
	    E >= 2
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, B + (C + J)), G),
	theory(y, [D = J], K, L, E, (F, ['not_='(A, B + (C + D))|G]), H, '_*'),
	'not_='(A, B + (C + J), M, I, (F, ['not_='(A, B + (C + D))|G]), H, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = I + A, D),
	theory(y, ['not_='(H, I + B)], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, I + A, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(A), H), E),
	theory(y, [i(B) = H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(i(A), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(i(A), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(H), B), E),
	theory(y, [H = A], I, J, C, (D, ['not_='(i(A), B)|E]), F, '_*'),
	'not_='(i(H), B, K, G, (D, ['not_='(i(A), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, i(B)), E),
	theory(y, [H = i(A)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, i(B), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, i(H)), E),
	theory(y, [B = H], I, J, C, (D, ['not_='(A, i(B))|E]), F, '_*'),
	'not_='(A, i(H), K, G, (D, ['not_='(A, i(B))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(A)), H), E),
	theory(y, [i(i(B)) = H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(i(i(A)), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(i(i(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(H)), B), E),
	theory(y, [H = A], I, J, C, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	'not_='(i(i(H)), B, K, G, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, i(i(B))), E),
	theory(y, [H = i(i(A))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, i(i(B)), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(i(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, i(i(H))), E),
	theory(y, [B = H], I, J, C, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	'not_='(A, i(i(H)), K, G, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = i(A), D),
	theory(y, ['not_='(H, i(B))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, i(A), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = i(B), D),
	theory(y, ['not_='(H, i(A))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, i(B), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(i(A))), H), E),
	theory(y, [i(i(i(B))) = H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(i(i(i(A))), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(i(i(i(A))), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(i(H))), B), E),
	theory(y, [H = A], I, J, C, (D, ['not_='(i(i(i(A))), B)|E]), F, '_*'),
	'not_='(i(i(i(H))), B, K, G, (D, ['not_='(i(i(i(A))), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, i(i(i(B)))), E),
	theory(y, [H = i(i(i(A)))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, i(i(i(B))), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(i(i(B))), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, i(i(i(H)))), E),
	theory(y, [B = H], I, J, C, (D, ['not_='(A, i(i(i(B))))|E]), F, '_*'),
	'not_='(A, i(i(i(H))), K, G, (D, ['not_='(A, i(i(i(B))))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = i(i(A)), D),
	theory(y, ['not_='(H, i(i(B)))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, i(i(A)), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(i(i(B)) = H, D),
	theory(y, ['not_='(i(i(A)), H)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(i(i(B)), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(A), H), E),
	theory(y, [abs(B) = H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(abs(A), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(abs(A), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(H), B), E),
	theory(y, [H = A], I, J, C, (D, ['not_='(abs(A), B)|E]), F, '_*'),
	'not_='(abs(H), B, K, G, (D, ['not_='(abs(A), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, abs(B)), E),
	theory(y, [H = abs(A)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, abs(B), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, abs(H)), E),
	theory(y, [B = H], I, J, C, (D, ['not_='(A, abs(B))|E]), F, '_*'),
	'not_='(A, abs(H), K, G, (D, ['not_='(A, abs(B))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(A)), H), E),
	theory(y, [abs(abs(B)) = H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(abs(abs(A)), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(abs(abs(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(H)), B), E),
	theory(y, [H = A], I, J, C, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	'not_='(abs(abs(H)), B, K, G, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, abs(abs(B))), E),
	theory(y, [H = abs(abs(A))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, abs(abs(B)), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(abs(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, abs(abs(H))), E),
	theory(y, [B = H], I, J, C, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	'not_='(A, abs(abs(H)), K, G, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = abs(A), D),
	theory(y, ['not_='(H, abs(B))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, abs(A), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = abs(B), D),
	theory(y, ['not_='(H, abs(A))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, abs(B), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(abs(A))), H), E),
	theory(y, [abs(abs(abs(B))) = H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(abs(abs(abs(A))), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(abs(abs(abs(A))), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(abs(H))), B), E),
	theory(y, [H = A], I, J, C, (D, ['not_='(abs(abs(abs(A))), B)|E]), F, '_*'),
	'not_='(abs(abs(abs(H))), B, K, G, (D, ['not_='(abs(abs(abs(A))), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, abs(abs(abs(B)))), E),
	theory(y, [H = abs(abs(abs(A)))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, abs(abs(abs(B))), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(abs(abs(B))), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, abs(abs(abs(H)))), E),
	theory(y, [B = H], I, J, C, (D, ['not_='(A, abs(abs(abs(B))))|E]), F, '_*'),
	'not_='(A, abs(abs(abs(H))), K, G, (D, ['not_='(A, abs(abs(abs(B))))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = abs(abs(A)), D),
	theory(y, ['not_='(H, abs(abs(B)))], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, abs(abs(A)), K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(abs(abs(B)) = H, D),
	theory(y, ['not_='(abs(abs(A)), H)], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(abs(abs(B)), H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(i(i(A)))), H), E),
	theory(y, [i(i(B)) = I, i(i(I)) = H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(i(i(i(i(A)))), H, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(i(i(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(i(i(H)))), I), E),
	theory(y, [H = A, i(i(B)) = I], J, K, C, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	'not_='(i(i(i(i(H)))), I, L, G, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(i(i(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(i(i(H)))), B), E),
	theory(y, [H = I, i(i(I)) = A], J, K, C, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	'not_='(i(i(i(i(H)))), B, L, G, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, i(i(I))), E),
	theory(y, [i(i(B)) = I, H = i(i(i(i(A))))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, i(i(I)), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(i(i(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, i(i(B))), E),
	theory(y, [I = A, H = i(i(i(i(I))))], J, K, C, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	'not_='(H, i(i(B)), L, G, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(i(i(i(B)))), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, i(i(H))), E),
	theory(y, [B = I, i(i(I)) = H], J, K, C, (D, ['not_='(A, i(i(i(i(B)))))|E]), F, '_*'),
	'not_='(A, i(i(H)), L, G, (D, ['not_='(A, i(i(i(i(B)))))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = i(i(i(A))), D),
	theory(y, [i(i(B)) = I, 'not_='(H, i(I))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, i(i(i(A))), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(i(i(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = i(i(i(I))), D),
	theory(y, [I = A, 'not_='(H, i(B))], J, K, C, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	=(H, i(i(i(I))), L, G, (D, ['not_='(i(i(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(H)), I), E),
	theory(y, [H = i(i(A)), i(i(i(i(B)))) = I], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(i(i(H)), I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(i(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(A)), H), E),
	theory(y, [B = I, i(i(i(i(I)))) = H], J, K, C, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	'not_='(i(i(A)), H, L, G, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(i(i(i(i(A)))), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(i(i(H)), B), E),
	theory(y, [I = A, H = i(i(I))], J, K, C, (D, ['not_='(i(i(i(i(A)))), B)|E]), F, '_*'),
	'not_='(i(i(H)), B, L, G, (D, ['not_='(i(i(i(i(A)))), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, i(i(i(i(B))))), E),
	theory(y, [I = i(i(A)), H = i(i(I))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, i(i(i(i(B)))), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(i(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, i(i(i(i(I))))), E),
	theory(y, [B = I, H = i(i(A))], J, K, C, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	'not_='(H, i(i(i(i(I)))), L, G, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(i(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, i(i(i(i(H))))), E),
	theory(y, [I = H, B = i(i(I))], J, K, C, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	'not_='(A, i(i(i(i(H)))), L, G, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = i(I), D),
	theory(y, [I = i(i(A)), 'not_='(H, i(i(i(B))))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, i(I), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, i(i(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = i(A), D),
	theory(y, [B = I, 'not_='(H, i(i(i(I))))], J, K, C, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	=(H, i(A), L, G, (D, ['not_='(A, i(i(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(abs(abs(A)))), H), E),
	theory(y, [abs(abs(B)) = I, abs(abs(I)) = H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(abs(abs(abs(abs(A)))), H, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(abs(abs(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(abs(abs(H)))), I), E),
	theory(y, [H = A, abs(abs(B)) = I], J, K, C, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	'not_='(abs(abs(abs(abs(H)))), I, L, G, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(abs(abs(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(abs(abs(H)))), B), E),
	theory(y, [H = I, abs(abs(I)) = A], J, K, C, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	'not_='(abs(abs(abs(abs(H)))), B, L, G, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, abs(abs(I))), E),
	theory(y, [abs(abs(B)) = I, H = abs(abs(abs(abs(A))))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, abs(abs(I)), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(abs(abs(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, abs(abs(B))), E),
	theory(y, [I = A, H = abs(abs(abs(abs(I))))], J, K, C, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	'not_='(H, abs(abs(B)), L, G, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(abs(abs(abs(B)))), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, abs(abs(H))), E),
	theory(y, [B = I, abs(abs(I)) = H], J, K, C, (D, ['not_='(A, abs(abs(abs(abs(B)))))|E]), F, '_*'),
	'not_='(A, abs(abs(H)), L, G, (D, ['not_='(A, abs(abs(abs(abs(B)))))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = abs(abs(abs(A))), D),
	theory(y, [abs(abs(B)) = I, 'not_='(H, abs(I))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, abs(abs(abs(A))), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(abs(abs(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = abs(abs(abs(I))), D),
	theory(y, [I = A, 'not_='(H, abs(B))], J, K, C, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	=(H, abs(abs(abs(I))), L, G, (D, ['not_='(abs(abs(A)), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(H)), I), E),
	theory(y, [H = abs(abs(A)), abs(abs(abs(abs(B)))) = I], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(abs(abs(H)), I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(abs(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(A)), H), E),
	theory(y, [B = I, abs(abs(abs(abs(I)))) = H], J, K, C, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	'not_='(abs(abs(A)), H, L, G, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(abs(abs(abs(abs(A)))), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(abs(abs(H)), B), E),
	theory(y, [I = A, H = abs(abs(I))], J, K, C, (D, ['not_='(abs(abs(abs(abs(A)))), B)|E]), F, '_*'),
	'not_='(abs(abs(H)), B, L, G, (D, ['not_='(abs(abs(abs(abs(A)))), B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, abs(abs(abs(abs(B))))), E),
	theory(y, [I = abs(abs(A)), H = abs(abs(I))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, abs(abs(abs(abs(B)))), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(abs(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, abs(abs(abs(abs(I))))), E),
	theory(y, [B = I, H = abs(abs(A))], J, K, C, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	'not_='(H, abs(abs(abs(abs(I)))), L, G, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(abs(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, abs(abs(abs(abs(H))))), E),
	theory(y, [I = H, B = abs(abs(I))], J, K, C, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	'not_='(A, abs(abs(abs(abs(H)))), L, G, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = abs(I), D),
	theory(y, [I = abs(abs(A)), 'not_='(H, abs(abs(abs(B))))], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=(H, abs(I), L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, abs(abs(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = abs(A), D),
	theory(y, [B = I, 'not_='(H, abs(abs(abs(I))))], J, K, C, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	=(H, abs(A), L, G, (D, ['not_='(A, abs(abs(B)))|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(H, B), E),
	theory(y, [H =< A, A =< H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	'not_='(H, B, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, ['not_='(H, B), A =< H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, A, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, ['not_='(H, B), H =< A], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(A, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, ['not_='(H, A), B =< H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, B, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< H, D),
	theory(y, ['not_='(H, A), H =< B], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(B, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, ['not_='(B, H), A =< H], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, A, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, ['not_='(B, H), H =< A], I, J, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(A, H, K, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [B = I, 'not_='(H, I), A =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, A, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [I = A, 'not_='(H, B), I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [B = I, 'not_='(H, I), H =< A], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(A, H, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [H = A, 'not_='(I, B), I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, ['not_='(H, A), I = B, I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [I = B, 'not_='(H, I), A =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, A, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, ['not_='(I, A), H = B, I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [I = B, 'not_='(H, I), H =< A], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(A, H, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, ['not_='(B, H), I = A, I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, ['not_='(I, H), B = I, A =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, A, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, ['not_='(B, I), H = A, I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, ['not_='(I, H), B = I, H =< A], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(A, H, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [I =< A, A =< I, 'not_='(H, B), I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [H =< A, A =< H, 'not_='(I, B), I =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [B =< I, I =< B, 'not_='(H, I), A =< H], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(H, A, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
'not_='(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [B =< I, I =< B, 'not_='(H, I), H =< A], J, K, C, (D, ['not_='(A, B)|E]), F, '_*'),
	=<(A, H, L, G, (D, ['not_='(A, B)|E]), F, '_*'),
	p_count(theory).
=(A, B, r, C, (D, E), F, '_*') :-
	!,
	p_never_identical(A = B, D),
	=(A, B, n, C, (D, E), F, '_*').
=(A, B, n, C, (D, E), F, '_*') :-
	p_identical('not_='(A, B), E),
	!,
	incval(inf),
	p_count(red_cut).
=(A, B, n, C, D, (E, F), '_*') :-
	p_identical(A = B, E),
	!,
	incval(inf),
	p_count(factor_cut).
=(A, B, n, C, (D, E), F, '_*') :-
	p_unifiable('not_='(A, B), E),
	incval(inf),
	p_count(red).
=(A + B, B + A, n, C, D, E, '_*') :-
	incval(inf),
	p_count(ext).
=(A + 0, A, n, B, C, D, '_*') :-
	incval(inf),
	p_count(ext).
=(A + i(A), 0, n, B, C, D, '_*') :-
	incval(inf),
	p_count(ext).
=(A, A, n, B, (C, D), E, '_*') :-
	incval(inf),
	theory(y, [], F, G, B, ([A = A|C], D), E, '_*'),
	p_count(theory).
=(A + (B + C), A + B + C, n, D, (E, F), G, '_*') :-
	incval(inf),
	theory(y, [], H, I, D, ([A + (B + C) = A + B + C|E], F), G, '_*'),
	p_count(theory).
=(A + B + C, A + (B + C), n, D, (E, F), G, '_*') :-
	incval(inf),
	theory(y, [], H, I, D, ([A + B + C = A + (B + C)|E], F), G, '_*'),
	p_count(theory).
=(A + (B + (C + D)), A + B + C + D, n, E, (F, G), H, '_*') :-
	incval(inf),
	theory(y, [], I, J, E, ([A + (B + (C + D)) = A + B + C + D|F], G), H, '_*'),
	p_count(theory).
=(abs(A), A, n, B, (C, D), E, '_*') :-
	(
	    B >= 1
	->
	    plus(F, 1, B)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	=<(0, A, r, F, ([abs(A) = A|C], D), E, '_*'),
	p_count(ext).
=(abs(A), i(A), n, B, (C, D), E, '_*') :-
	(
	    B >= 1
	->
	    plus(F, 1, B)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	=<(A, 0, r, F, ([abs(A) = i(A)|C], D), E, '_*'),
	p_count(ext).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B = A, D),
	theory(y, [], H, I, C, ([A = B|D], E), F, '_*'),
	=(B, A, J, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A + B, C + B, n, D, (E, F), G, '_*') :-
	(
	    D >= 1
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = C, E),
	theory(y, [], I, J, D, ([A + B = C + B|E], F), G, '_*'),
	=(A, C, K, H, ([A + B = C + B|E], F), G, '_*'),
	p_count(theory).
=(A + B, A + C, n, D, (E, F), G, '_*') :-
	(
	    D >= 1
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B = C, E),
	theory(y, [], I, J, D, ([A + B = A + C|E], F), G, '_*'),
	=(B, C, K, H, ([A + B = A + C|E], F), G, '_*'),
	p_count(theory).
=(i(A), i(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = B, D),
	theory(y, [], H, I, C, ([i(A) = i(B)|D], E), F, '_*'),
	=(A, B, J, G, ([i(A) = i(B)|D], E), F, '_*'),
	p_count(theory).
=(abs(A), abs(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = B, D),
	theory(y, [], H, I, C, ([abs(A) = abs(B)|D], E), F, '_*'),
	=(A, B, J, G, ([abs(A) = abs(B)|D], E), F, '_*'),
	p_count(theory).
=(A, B + (C + D), n, E, (F, G), H, '_*') :-
	(
	    E >= 1
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = B + C + D, F),
	theory(y, [], J, K, E, ([A = B + (C + D)|F], G), H, '_*'),
	=(A, B + C + D, L, I, ([A = B + (C + D)|F], G), H, '_*'),
	p_count(theory).
=(A, B + C + D, n, E, (F, G), H, '_*') :-
	(
	    E >= 1
	->
	    plus(I, 1, E)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = B + (C + D), F),
	theory(y, [], J, K, E, ([A = B + C + D|F], G), H, '_*'),
	=(A, B + (C + D), L, I, ([A = B + C + D|F], G), H, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = H, D),
	theory(y, [B = H], I, J, C, ([A = B|D], E), F, '_*'),
	=(A, H, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B = H, D),
	theory(y, [H = A], I, J, C, ([A = B|D], E), F, '_*'),
	=(B, H, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B + C, n, D, (E, F), G, '_*') :-
	(
	    D >= 2
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = I + C, E),
	theory(y, [I = B], J, K, D, ([A = B + C|E], F), G, '_*'),
	=(A, I + C, L, H, ([A = B + C|E], F), G, '_*'),
	p_count(theory).
=(A, B + C, n, D, (E, F), G, '_*') :-
	(
	    D >= 2
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = B + I, E),
	theory(y, [I = C], J, K, D, ([A = B + C|E], F), G, '_*'),
	=(A, B + I, L, H, ([A = B + C|E], F), G, '_*'),
	p_count(theory).
=(A, i(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = i(H), D),
	theory(y, [H = B], I, J, C, ([A = i(B)|D], E), F, '_*'),
	=(A, i(H), K, G, ([A = i(B)|D], E), F, '_*'),
	p_count(theory).
=(A, i(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = i(H), D),
	theory(y, [B = H], I, J, C, ([A = i(B)|D], E), F, '_*'),
	=(A, i(H), K, G, ([A = i(B)|D], E), F, '_*'),
	p_count(theory).
=(A, i(i(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = i(i(H)), D),
	theory(y, [H = B], I, J, C, ([A = i(i(B))|D], E), F, '_*'),
	=(A, i(i(H)), K, G, ([A = i(i(B))|D], E), F, '_*'),
	p_count(theory).
=(i(i(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(i(i(H)) = B, D),
	theory(y, [A = H], I, J, C, ([i(i(A)) = B|D], E), F, '_*'),
	=(i(i(H)), B, K, G, ([i(i(A)) = B|D], E), F, '_*'),
	p_count(theory).
=(A, abs(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = abs(H), D),
	theory(y, [H = B], I, J, C, ([A = abs(B)|D], E), F, '_*'),
	=(A, abs(H), K, G, ([A = abs(B)|D], E), F, '_*'),
	p_count(theory).
=(A, abs(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = abs(H), D),
	theory(y, [B = H], I, J, C, ([A = abs(B)|D], E), F, '_*'),
	=(A, abs(H), K, G, ([A = abs(B)|D], E), F, '_*'),
	p_count(theory).
=(A, abs(abs(B)), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = abs(abs(H)), D),
	theory(y, [H = B], I, J, C, ([A = abs(abs(B))|D], E), F, '_*'),
	=(A, abs(abs(H)), K, G, ([A = abs(abs(B))|D], E), F, '_*'),
	p_count(theory).
=(abs(abs(A)), B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(abs(abs(H)) = B, D),
	theory(y, [A = H], I, J, C, ([abs(abs(A)) = B|D], E), F, '_*'),
	=(abs(abs(H)), B, K, G, ([abs(abs(A)) = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< B, D),
	theory(y, [B =< A], H, I, C, ([A = B|D], E), F, '_*'),
	=<(A, B, J, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [A =< B], H, I, C, ([A = B|D], E), F, '_*'),
	=<(B, A, J, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, i(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = i(i(i(H))), D),
	theory(y, [H = I, i(i(I)) = B], J, K, C, ([A = i(B)|D], E), F, '_*'),
	=(A, i(i(i(H))), L, G, ([A = i(B)|D], E), F, '_*'),
	p_count(theory).
=(A, i(i(i(B))), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = i(H), D),
	theory(y, [I = B, H = i(i(I))], J, K, C, ([A = i(i(i(B)))|D], E), F, '_*'),
	=(A, i(H), L, G, ([A = i(i(i(B)))|D], E), F, '_*'),
	p_count(theory).
=(A, abs(B), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = abs(abs(abs(H))), D),
	theory(y, [H = I, abs(abs(I)) = B], J, K, C, ([A = abs(B)|D], E), F, '_*'),
	=(A, abs(abs(abs(H))), L, G, ([A = abs(B)|D], E), F, '_*'),
	p_count(theory).
=(A, abs(abs(abs(B))), n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = abs(H), D),
	theory(y, [I = B, H = abs(abs(I))], J, K, C, ([A = abs(abs(abs(B)))|D], E), F, '_*'),
	=(A, abs(H), L, G, ([A = abs(abs(abs(B)))|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A = H, D),
	theory(y, [H =< B, B =< H], I, J, C, ([A = B|D], E), F, '_*'),
	=(A, H, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [H = B, H =< A], I, J, C, ([A = B|D], E), F, '_*'),
	=<(A, H, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [H = B, A =< H], I, J, C, ([A = B|D], E), F, '_*'),
	=<(H, A, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [B = H, H =< A], I, J, C, ([A = B|D], E), F, '_*'),
	=<(A, H, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [B = H, A =< H], I, J, C, ([A = B|D], E), F, '_*'),
	=<(H, A, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< H, D),
	theory(y, [H = A, H =< B], I, J, C, ([A = B|D], E), F, '_*'),
	=<(B, H, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, [H = A, B =< H], I, J, C, ([A = B|D], E), F, '_*'),
	=<(H, B, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [H = I, I = B, H =< A], J, K, C, ([A = B|D], E), F, '_*'),
	=<(A, H, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [H = I, I = B, A =< H], J, K, C, ([A = B|D], E), F, '_*'),
	=<(H, A, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [H =< B, B =< H, H =< A], I, J, C, ([A = B|D], E), F, '_*'),
	=<(A, H, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [H =< B, B =< H, A =< H], I, J, C, ([A = B|D], E), F, '_*'),
	=<(H, A, K, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [B = I, H = I, H =< A], J, K, C, ([A = B|D], E), F, '_*'),
	=<(A, H, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [B = I, H = I, A =< H], J, K, C, ([A = B|D], E), F, '_*'),
	=<(H, A, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< H, D),
	theory(y, [I = A, H = I, H =< B], J, K, C, ([A = B|D], E), F, '_*'),
	=<(B, H, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, [I = A, H = I, B =< H], J, K, C, ([A = B|D], E), F, '_*'),
	=<(H, B, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [H =< I, I =< H, I = B, H =< A], J, K, C, ([A = B|D], E), F, '_*'),
	=<(A, H, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [H =< I, I =< H, I = B, A =< H], J, K, C, ([A = B|D], E), F, '_*'),
	=<(H, A, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [I =< B, B =< I, H = I, H =< A], J, K, C, ([A = B|D], E), F, '_*'),
	=<(A, H, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [I =< B, B =< I, H = I, A =< H], J, K, C, ([A = B|D], E), F, '_*'),
	=<(H, A, L, G, ([A = B|D], E), F, '_*'),
	p_count(theory).
=<(A, B, r, C, (D, E), F, '_*') :-
	!,
	p_never_identical(A =< B, D),
	=<(A, B, n, C, (D, E), F, '_*').
=<(A, B, n, C, (D, E), F, '_*') :-
	p_identical('not_=<'(A, B), E),
	!,
	incval(inf),
	p_count(red_cut).
=<(A, B, n, C, D, (E, F), '_*') :-
	p_identical(A =< B, E),
	!,
	incval(inf),
	p_count(factor_cut).
=<(A, B, n, C, (D, E), F, '_*') :-
	p_unifiable('not_=<'(A, B), E),
	incval(inf),
	p_count(red).
=<(0, x + y, n, A, B, C, '_*') :-
	incval(inf),
	p_count(ext).
=<(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	'not_=<'(B, A, r, G, ([A =< B|D], E), F, '_*'),
	p_count(ext).
=<(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	'not_=<'(B, A, r, G, ([A =< B|D], E), F, '_*'),
	p_count(ext).
=<(A + B, C + B, n, D, (E, F), G, '_*') :-
	(
	    D >= 1
	->
	    plus(H, 1, D)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	=<(A, C, r, H, ([A + B =< C + B|E], F), G, '_*'),
	p_count(ext).
=<(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, [H = A], I, J, C, ([A =< B|D], E), F, '_*'),
	=<(H, B, K, G, ([A =< B|D], E), F, '_*'),
	p_count(theory).
=<(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [H = B], I, J, C, ([A =< B|D], E), F, '_*'),
	=<(A, H, K, G, ([A =< B|D], E), F, '_*'),
	p_count(theory).
=<(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, [A = H], I, J, C, ([A =< B|D], E), F, '_*'),
	=<(H, B, K, G, ([A =< B|D], E), F, '_*'),
	p_count(theory).
=<(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [B = H], I, J, C, ([A =< B|D], E), F, '_*'),
	=<(A, H, K, G, ([A =< B|D], E), F, '_*'),
	p_count(theory).
=<(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [H =< B], I, J, C, ([A =< B|D], E), F, '_*'),
	=<(A, H, K, G, ([A =< B|D], E), F, '_*'),
	p_count(theory).
'not_=<'(A, B, r, C, (D, E), F, '_*') :-
	!,
	p_never_identical('not_=<'(A, B), E),
	'not_=<'(A, B, n, C, (D, E), F, '_*').
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	p_identical(A =< B, D),
	!,
	incval(inf),
	p_count(red_cut).
'not_=<'(A, B, n, C, D, (E, F), '_*') :-
	p_identical('not_=<'(A, B), F),
	!,
	incval(inf),
	p_count(factor_cut).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	p_unifiable(A =< B, D),
	incval(inf),
	p_count(red).
'not_=<'(x + y, 0, n, A, B, C, '_*') :-
	incval(inf),
	p_count(ext).
'not_=<'(abs(x + y), abs(x) + abs(y), n, A, B, C, '_*') :-
	incval(inf),
	p_count(ext).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 1
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	'not_=<'(A + H, B + H, r, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(ext).
'not_=<'(0, A, n, B, (C, D), E, '_*') :-
	(
	    B >= 1
	->
	    plus(F, 1, B)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	'not_='(abs(A), A, r, F, (C, ['not_=<'(0, A)|D]), E, '_*'),
	p_count(ext).
'not_=<'(A, 0, n, B, (C, D), E, '_*') :-
	(
	    B >= 1
	->
	    plus(F, 1, B)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	'not_='(abs(A), i(A), r, F, (C, ['not_=<'(A, 0)|D]), E, '_*'),
	p_count(ext).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(H, B), E),
	theory(y, [A = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_=<'(H, B, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(A, H), E),
	theory(y, [B = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_=<'(A, H, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(H, B), E),
	theory(y, [H = A], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_=<'(H, B, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(A, H), E),
	theory(y, [H = B], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_=<'(A, H, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(A, H), E),
	theory(y, [B =< H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_=<'(A, H, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_=<'(H, B), E),
	theory(y, [H =< A], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_=<'(H, B, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, ['not_=<'(H, B)], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, B), E),
	theory(y, [B =< A], H, I, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_='(A, B, J, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(B, A), E),
	theory(y, [B =< A], H, I, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_='(B, A, J, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(B, A)], H, I, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, J, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(A, B)], H, I, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, J, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(A, H), E),
	theory(y, [B =< A, B = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_='(A, H, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical('not_='(B, H), E),
	theory(y, [B =< A, A = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	'not_='(B, H, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = A, D),
	theory(y, [B =< A, 'not_='(H, B)], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=(H, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H = B, D),
	theory(y, [B =< A, 'not_='(H, A)], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=(H, B, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(B, H), A = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(A, H), B = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [H = A, 'not_='(B, H)], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [H = B, 'not_='(A, H)], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(H, B), A = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(H, A), B = H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [A = H, H = I, 'not_='(B, I)], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [B = H, H = I, 'not_='(A, I)], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [B =< A, 'not_='(H, B), A =< H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, [B =< A, 'not_='(H, A), B =< H], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, B, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [A =< H, H =< A, 'not_='(B, H)], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [B =< A, 'not_='(H, B), H =< A], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(A, H, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< H, D),
	theory(y, [B =< A, 'not_='(H, A), H =< B], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, H, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [B =< H, H =< B, 'not_='(A, H)], I, J, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, K, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [H = I, 'not_='(B, H), A = I], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [H = I, 'not_='(A, H), B = I], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(H, B), I = H, A = I], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, ['not_='(H, A), I = H, B = I], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< A, D),
	theory(y, [B =< A, B = I, 'not_='(H, I), A =< H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< B, D),
	theory(y, [B =< A, A = I, 'not_='(H, I), B =< H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, B, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [A =< H, H =< A, H = I, 'not_='(B, I)], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(A =< H, D),
	theory(y, [B =< A, B = I, 'not_='(H, I), H =< A], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(A, H, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< H, D),
	theory(y, [B =< A, A = I, 'not_='(H, I), H =< B], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, H, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [B =< H, H =< B, H = I, 'not_='(A, I)], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [B =< A, 'not_='(H, B), I = A, I =< H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [B =< A, 'not_='(H, A), I = B, I =< H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [H =< I, I =< H, 'not_='(B, I), A = H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [B =< A, 'not_='(I, B), H = A, I =< H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(H =< I, D),
	theory(y, [B =< A, 'not_='(I, A), H = B, I =< H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(H, I, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).
'not_=<'(A, B, n, C, (D, E), F, '_*') :-
	(
	    C >= 2
	->
	    plus(G, 1, C)
	;
	    setval(more_depth, 1),
	    !,
	    fail
	),
	incval(inf),
	p_never_identical(B =< A, D),
	theory(y, [H =< I, I =< H, 'not_='(A, I), B = H], J, K, C, (D, ['not_=<'(A, B)|E]), F, '_*'),
	=<(B, A, L, G, (D, ['not_=<'(A, B)|E]), F, '_*'),
	p_count(theory).

:- set_flag( variable_names, check_singletons).
