
/*
 * FDP - A First-Order Davis-Putnam proof procedure
 *
 * Version 0.9.4 (01/07/1999)
 *
 * Author: Peter Baumgartner, Universit"at Koblenz-Landau, 
 *         peter@uni-koblenz.de
 *
 * Usage: fdpll FILE
 *
 * where FILE is the input file in tme-Format (PROTEIN-Syntax) without
 * the extension .tme
 *
 * Flags:
 * The input file may contain flag settings of the form
 *          dp_flag(FLAG,VALUE).
 * e.g.
 *          dp_flag(term_depth,(3,2)).
 *          dp_flag(verbose,off).
 * please see the README for valid flag settings
 *
 * When fdp terminates, it either reports
 *          REFUTATION FOUND
 * which indicates that the clause set in FILE is unsatisfiable,
 * or it reports
 *          SATISFIABLE
 * and prints a list of literals (the special cases +X or -X are allowed as 
 * well); the literal sign is indicated
 * by + (-) for positive (negative) literals.
 * This list represents a model for the clause set in FILE
 * e.g. the list
 *
 *   [-X,+p(X,Y),-p(X,X),+p(a,a)]
 * 
 * In order to find out whether a ground atom is true in such an 
 * interpretation, locate the _most specific generalisation_ (MSG) of it,
 * disregarding the sign, and then use the sign to determine whether it
 * is true or false.
 * E.g. p(a,b) is true because the MSG is +p(X,Y), and the sign is +
 * E.g. p(b,b) is false because the MSG is -p(X,X), and the sign is -
 * E.g. p(a,a) is true because the MSG is +p(a,a), and the sign is +
 * E.g. q(a,a) is false because the MSG is -X, and the sign is -
 *
 * Please see the README for a more detailled explanation of the calculus.
 *
 */

%% This file contains the surrounding procedures
%% The real work is done in interpretation.pl

:- set_flag(occur_check, on).
:- set_flag(debug_compile, off).
:- [misc,parser,counters,interpretation].
:- lib(lists).
:- set_flag(output_mode, "V").

%% Flags are set by Prolog facts dp_flag(++Flag,+Value)
:- mode dp_flag(++,?).

%% Currently implemented Flags - please see the README file


%% node(L,Kind,Sibling,RelAnc,Level+SubLevel,Complexity)
%% Main data structure - the node in the tree
%% Meaning:
%%  L - the literal label of the node
%%  Kind - either the 'atom' univ or the atom 'pattern'
%%         univ means 'universal' literal, i.e. the interpretation along 
%%         the branch containing L contains all instances of L and cannot be
%%         updated.
%%         pattern means that the interpretation along 
%%         the branch containing L contains all instances of L, but 
%%         exceptions are possible by adding strict instances of the 
%%         complement of L that define 'exceptions' to the ground instances
%%         of L.
%% Sibling - the literal label of the right sibling of node, if it exists. 
%%         If K is that label then Sibling is [K], if the right
%%         sibling does not exist, then Sibling is []
%% RelAnc - The Relevant Ancestors of node. I.e. those ancestors from
%%         which node immediately depends on. CAUTIONS: RelAnc is not
%%         transitively closed, i.e. the 'RelAnc of the RelAncs' are not
%%         explicitly contained in RelAncs.
%% Level+SubLevel - Level indicates the depth of a node (counting from the
%%         root) that was used for splitting (binary branching in the tree).
%%         Sublevel numbers the unit-extensions done immediately below
%%         the split at level Level
%% Complexity - currently unused - contains a dummy value.
%%

%% Every node n has a counter associated which is a pair <Level,SubLevel>
%% The root has the value <0,0>
%% When going down a branch, Level is increased by one for every 
%% split, and Sublevel is increased for every unit conclusion below
%% this Level. Below, this pair is represented by the Prolog term
%% Level+SubLevel

%% We need some global variables. The constant 1000 limits the maximal
%% tree depth wrt splits (there is no limit on unit extensions) 
%% Please adjust to your needs.
:-  make_array(sublevel(1000),integer).
    % The number of unit conclusions, for given Level

:- dynamic(node/6).
:- mode node(?,?,?,?,?,?).

prove_file(FileName,EQTrafoBin,Flags) :-
	set_flag(occur_check, on),
	set_flag(output_mode, "V"),
	retract_all(dp_flag(_,_)),
	setval(file_name,FileName),
	setval(orig_file_name,FileName),
	init_parser(true),
	%% Set default flag values:
	assert(dp_flag(initial_interpretation,[+_X])),
	assert(dp_flag(verbose,on)),
	assert(dp_flag(eqt,off)),
	assert(dp_flag(automatic,off)),
	%% Not sure about the SOUNDNESS of this flag, so better don't touch it
	assert(dp_flag(univ_split,off)), 
	assert(dp_flag(time_limit,0)), % no time limit
	assert(dp_flag(restart_time,0)), % no restart time
	assert(dp_flag(eqax_restart_time,0)), % no restart time
	assert(dp_flag(level_cut,on)),
	assert(dp_flag(unit_extend,unset)),
	assert(dp_flag(back_subsumption,on)),
	%% Seems to work quite well in the average:
	assert(dp_flag(term_depth,(3,2))),
	consult_file(FileName),
	% default flag setting:
	( dp_flag(unit_extend,unset) ->
	  %% Playground...
	  ( horn_clause_set ->
%	      tr_dp(dp_flag(unit_extend,positive))
	      tr_dp(dp_flag(unit_extend,on))
	    ; 
%	      tr_dp(dp_flag(unit_extend,positive))
	      tr_dp(dp_flag(unit_extend,on))
	  )
        ; true
	),
	( member(quiet,Flags) ->
	  tr_dp(dp_flag(verbose,off))
	; true 
        ),
	( member(eqt,Flags) ->
	  tr_dp(dp_flag(eqt,on))
	; true 
        ),
	( member(automatic,Flags) ->
	  tr_dp(dp_flag(automatic,on))
	; true 
        ),
	( ( member(time_limit(_TL),Flags), number(_TL)) ->
	  tr_dp(dp_flag(time_limit,_TL))
	; true 
        ),
	( ( member(restart_time(_RT),Flags), number(_RT)) ->
	  tr_dp(dp_flag(restart_time,_RT))
	; true 
        ),
	( ( member(eqax_restart_time(_EQAXRT),Flags), number(_EQAXRT)) ->
	  tr_dp(dp_flag(eqax_restart_time,_EQAXRT))
	; true 
        ),
	( member(term_depth(_DINIT,_DINC),Flags) ->
	  ( number(_DINIT) -> 
	    DINIT = _DINIT 
	  ; dp_flag(term_depth,(DINIT,_))
          ),
	  ( number(_DINC) -> 
	    DINC = _DINC 
	  ; dp_flag(term_depth,(_,DINC))
          ),
	  tr_dp(dp_flag(term_depth,(DINIT,DINC)))
	; true 
        ),
	%% see if we have equality clause set
	( equality_clause_set -> 
	  setval(equality,true),
	  ( dp_flag(automatic,on) -> tr_dp(dp_flag(eqt,on)) ; true)
        ; setval(equality,false)
        ),
	%% get the init value of the term_depth flag, and make it active:
	dp_flag(term_depth,(Init,_Inc)),
	set_term_depth_max(Init),
	( dp_flag(eqt,on) ->
	  call_eqt(FileName,EQTrafoBin),
	  setval(proving_phase,'with equality transformation applied')
	; setval(proving_phase,'as initially given')
        ),
	writeln('FDPLL - A First-Order Davis-Putnam Theorem Prover'),
        writeln('Version 0.9.4 (01/07/1999)'),
	garbage_collect,
	statistics(times, [StartTime, _, _]),
	setval(start_time,StartTime),
	setval(term_depth_inc_time,StartTime),
	set_time_limits,
	set_interrupt_handler(vtalrm, fdp_alarm_handler/1),
	set_timer(virtual,1), 
	setval(aborted,false),
	setval(refuted,false),
	setval(satisfiable,false),
	setval(restart_count,0),
	start_proving,
	my_print_statistics.

call_eqt(FileName,EQTrafoBin) :-
	concat_atoms(EQTrafoBin,' ',E1),
	concat_atoms(E1,FileName,E2),
	( system(E2) ->
	  %% call succeeded
	  init_parser(false),
	  concat_atoms(FileName,'-eqt',FileNameEqt),
	  consult_file(FileNameEqt),
	  setval(file_name,FileNameEqt)
	; tr_dp(eqt,off) %% remember that eqt was not called
        ).

set_time_limits :-
	%% can be sure here that times are nummbers, but never 'unset'
	dp_flag(time_limit,TL),
	( TL = 0 ->
	  TicksToAbort is 1000000 % some large number
	; TicksToAbort is fix(TL)
        ),
	setval(ticks_to_abort,TicksToAbort),
	setval(saved_ticks_to_abort,TicksToAbort),

	%%%%% Compute ticks to equality axioms restart %%%%%%

	( ( getval(equality, false) ; dp_flag(eqt,off) ) ->
	  %% no point in restarting with axioms, or
	  %% or will not call eqtrafo, so start off with axioms
	  setval(ticks_to_eqax_restart,TicksToAbort)
	; %% so, equality literals are present, and eqtrafo is to be called
	  %% This is the only situation that makes sense for restart with
	  %% axioms
	  dp_flag(eqax_restart_time,EQAX_RT),
	  ( EQAX_RT = 0 ->
	    %% use default values
	    ( dp_flag(automatic,on) ->
		( getval(ticks_to_abort,1000000) ->
		  %% unspecified, take 3 minutes
	          TicksToEqaxRestart is 150
	        ; %% else use a fraction of the given time limit
	          REAL_EQAX_RT is TicksToAbort * 0.75,
	          TicksToEqaxRestart is fix(REAL_EQAX_RT)
	        )
	    ; %% non-autmatic mode
	      TicksToEqaxRestart is TicksToAbort
	    )
	  ; %% EQAX_RT is different from default setting 0    
	    integer(EQAX_RT) ->
	    %% Immediately given
	    TicksToEqaxRestart is EQAX_RT
	  ; ( real(EQAX_RT), 0.0 < EQAX_RT, EQAX_RT =< 1.0 ) ->
	    %% valid range, take this portion of total timelimit
	    ( getval(ticks_to_abort,1000000) ->
	      %% unspecified:
	      fdp_fatal_error('Float value of eqax_restart_time requires explicitly given time limit',[])
	    ; REAL_EQAX_RT is TicksToAbort * EQAX_RT,
	      TicksToEqaxRestart is fix(REAL_EQAX_RT)
	    )
	  ; real(EQAX_RT) ->
	    fdp_fatal_error('Float value of eqax_restart_time must be in range ]0..1[',[])
	  ; fdp_fatal_error('eqax_restart_time must be a float or a number',[])
          ),
	  setval(ticks_to_eqax_restart,TicksToEqaxRestart)
        ),
	getval(ticks_to_eqax_restart,CTicksToEqaxRestart),
	%% save the value just for printing of flags at the end:
	setval(saved_ticks_to_eqax_restart,CTicksToEqaxRestart),

	%%%%% Compute ticks to restart %%%%%%
        set_ticks_to_restart(CTicksToEqaxRestart,true).

set_ticks_to_restart(Limit,AbortOnProblem) :- 
	( function_free_clause_set ->
	  %% no point in restarting with a higher term depth
	  setval(ticks_to_restart,Limit)
        ; dp_flag(restart_time,RT),
	  ( RT = 0  ->
	    %% use defaults
	    ( dp_flag(automatic,on) ->
	      ( Limit = 1000000 ->
		  %% unspecified or not computed, take 2 minutes
	        TicksToRestart is 100
	      ; %% else use a fraction of the given time limit
	        REAL_RT is Limit * 0.7,
	        TicksToRestart is fix(REAL_RT)
	        )
	    ; %% non-autmatic mode
	      TicksToRestart is Limit
	    )
	  ; integer(RT) ->
	    %% Immediately given
	    TicksToRestart is RT
	  ; ( real(RT), 0.0 < RT, RT < 1.0 ) ->
	    %% valid range, take this portion of total timelimit
	    ( ( Limit = 1000000, AbortOnProblem = true) ->
	      %% unspecified:
	      fdp_fatal_error('Float value of restart_time requires explicitly given time limit or eqax_restart_time',[])
	    ; Limit = 1000000 ->
	      %% so AbortOnProblem = false, take last ressort
	      TicksToRestart is 100
	    ; REAL_RT is Limit * RT,
	      TicksToRestart is fix(REAL_RT)
	    )
	  ; real(RT) ->
	    fdp_fatal_error('Float value of restart_time must be in range ]0..1[',[])
	  ; fdp_fatal_error('restart_time must be a float or a number',[])
          ),
	  setval(ticks_to_restart,TicksToRestart)
        ),
	getval(ticks_to_restart,CTicksToRestart),
	%% save the value just for printing of flags at the end:
	setval(saved_ticks_to_restart,CTicksToRestart).


fdp_fatal_error(Format,Args) :-
	printf('%nERROR: ',[]),
	printf(Format,Args),
	nl,nl,
	abort.


start_proving :-
	getval(file_name,FileName),
	getval(restart_count,RC),
	( RC = 0 ->
	  printf('Proving %w ...%n',[FileName])
        ; getval(term_depth_max,M),
	  printf('Proving %w (restart with term depth limit %w) ...%n',
                 [FileName,M])
	),
        flush(user),
	%% Reset the statistics counters:
	setval(back_subsume_count,0),
	setval(branch_count,0),
	setval(split_count,0),
	setval(unit_count,0),
	setval(uniquify_count,0),
	setval(level_cut_count,0),
	init_interpretation,
	%% Here the workhorse is called:
	!,
	block( (satisfy ; true), abort_fdp, fdp_block_exit_handler ).

fdp_block_exit_handler :-
	getval(ticks_to_abort,0),
	!, 
	%% so have to really abort
	set_timer(virtual,0), % switch it off
	setval(aborted,true).
fdp_block_exit_handler :-
	%% here we know that ticks_to_abort is greater than 0,
	%% hence ticks_to_restart or ticks_to_eqax_restart must be 0,
	getval(ticks_to_eqax_restart,0),
	!, 
	set_timer(virtual,0), %% stop timer for the time being.

	%% never want this restart again:
	setval(ticks_to_eqax_restart,1000000), 

	%% read in the original file again
	init_parser(false),
	getval(orig_file_name,FileName),
	consult_file(FileName),
	%% remember the file we are currently working on
	setval(file_name,FileName),

	%% compute the restart interval again:
	getval(saved_ticks_to_abort,STTA),
	( STTA = 1000000 ->
	  %% so have no limit
	  Limit = 1000000 
        ; % otherwise take what is left over
	  getval(ticks_to_abort,TTA),
	  Limit = TTA
        ),
	set_ticks_to_restart(Limit,false),

	%% Have 0 restarts so far in the freshly begun session:
	setval(restart_count,0),
	%% reset the initial term depth to the given value
	dp_flag(term_depth,(Init,_Inc)),
	set_term_depth_max(Init),

	%% counters have to be explicitly initialised:
	( \+ dp_flag(unit_extend,off) ->
	  univ_extend_ctr_clear
        ; true
        ),
	setval(proving_phase,'as initially given, after non-success with equality transformation'),
	%% restart the timer
	set_interrupt_handler(vtalrm, fdp_alarm_handler/1),
	set_timer(virtual,1), 
	start_proving.

fdp_block_exit_handler :-
	%% here we know that ticks_to_abort is greater than 0,
	%% and that ticks_to_eqax_restart is greater than 0
	%% hence ticks_to_restart must be 0,
	%% further, function_free_clause_set must be false
        %% initialise the restart counter again:
	getval(saved_ticks_to_restart,TTR),
	setval(ticks_to_restart,TTR),
	%% reset the counters:
	%% univ_extend_ctr_clear,
	%% no - is part of this one:
	increase_term_depth_limit,
	%% getval(term_depth_max,TDM),
	incval(restart_count),
	% printf("RESTART with term depth limit %w%n",[TDM]),
	start_proving.

my_print_statistics :-
	statistics(times, [EndTime, _, _]), 
	getval(start_time,StartTime),
	Time is EndTime - StartTime, 
	writeln('Done.'), flush(user),
	%% Finally print out the statistics:
	getval(file_name,FileName),
	printf('Input File...............: %w.tme%n',[FileName]), flush(user),
	printf('System...................: ',[]), flush(user),
        sh('uname -a'),
	dp_flag(automatic,Automatic),
	printf('Automatic mode...........: %w%n',[Automatic]),
	printf('Time limit...............: ',[]), flush(user),
	dp_flag(time_limit,TimeLimit),
	( TimeLimit = 0 ->
	  printf("off%n",[])
	; printf("%w seconds%n",[TimeLimit])

        ),
	printf('Current restart interval.: ',[]), flush(user),
	getval(saved_ticks_to_eqax_restart,STTER),
	getval(saved_ticks_to_restart,STTR),
	( STTR = STTER ->
	  printf("off%n",[])
	; printf("%w seconds%n",[STTR])
        ),
	printf('Restart with =-axioms....: ',[]), flush(user),
	getval(saved_ticks_to_abort,STTA),
	( STTER = STTA ->
	  printf("off%n",[])
	; printf("%w seconds%n",[STTER])
        ),
	dp_flag(initial_interpretation,I),
	printf('Initial interpretation...: %w%n',[I]),
	printf('Clause set type..........: ',[]),
	( horn_clause_set ->
	    printf('Horn, ',[])
	;   printf('Non-Horn, ',[])
        ),
	( getval(equality,true) ->
	    printf('with equality%n',[])
	;   printf('without equality%n',[])
        ),
	dp_flag(eqt,EQTFlag),
	printf('Equality transformation..: %w%n',[EQTFlag]),
	printf('Non-constant functions...: ',[]),
	( function_free_clause_set ->
	    printf('no%n',[])
	;   printf('yes%n',[])
        ),
	dp_flag(term_depth,(InitTermDepth,IncTermDepth)),
	printf('Term depth settings......: %w/%w (Init/Increment)%n',[InitTermDepth,IncTermDepth]),
        printf('%nFinal tree statistics:%n',[]),
        getval(proving_phase,PPh),
	printf('Tree for clause set......: %w%n',[PPh]),
        getval(restart_count,RC),
	printf('# Restarts...............: %w%n',[RC]),
	getval(term_depth_max,TDM),
	printf('Term depth limit.........: %w%n',[TDM]),
	getval(split_count,SC),
	printf('# Splits.................: %w%n',[SC]),
%	getval(uniquify_count,UNC),
%	printf('# Uniquifiquations.......: %w%n',[UNC]),
	getval(unit_count,UC),
	printf('# Unit resulting steps...: %w%n',[UC]),
	getval(back_subsume_count,BSC),
	printf('# Unit back subsumptions.: %w%n',[BSC]),
	getval(branch_count,BC),
	printf('# Branches closed........: %w%n',[BC]),
	(dp_flag(level_cut,on) ->
	    getval(level_cut_count,LC)
	;   LC = '-'),
	printf('# Level cuts.............: %w%n',[LC]),
	printf('Time.....................: %w seconds.%n',[Time]), 
	printf('Result...................: ',[]), 
	( getval(aborted,true) ->
	  printf('ABORT due to time limit%n',[])
        ; getval(refuted,true) ->
	  printf('REFUTATION FOUND%n',[])
        ; getval(satisfiable,true) ->
	  branch_literals(Branch),
	  printf('SATISFIABLE with model %Dw%n',[Branch])
	; printf('UNKNOWN%n',[])
        ).

fdp_alarm_handler(_) :-   % 24 or 26 usually
	decval(ticks_to_abort),
	getval(ticks_to_abort,TTA),
	decval(ticks_to_restart),
	getval(ticks_to_restart,TTR),
	decval(ticks_to_eqax_restart),
	getval(ticks_to_eqax_restart,TTER),
	( ( TTA = 0 ; TTR = 0 ; TTER = 0 ) ->
	  exit_block(abort_fdp)
        ; true
        ).
%	dp_flag(time_limit,TL),
%	printf("Time limit (%w seconds) exhausted%n",[]),
%        abort.
%fdp_alarm_handler(X) :-
%	printf('Hugh? Got uncaught signal %w%n',[X]).



%% init_interpretation
%% set up the initial tableaux, consisting of a branch with literals
%% labeled by the initial interpretation (cf. respective flag above)
init_interpretation :-
	setval(branch_count,0),
	setval(node_ident,0),
	retract_all(node(_,_,_,_,_,_)),
	dp_flag(initial_interpretation,IGiven), 
	%% see if one of [-_] or [+_] is present. If not, add one
	( (delete(-X,IGiven,IPure), var(X)) ->
	    I = [-X|IPure]
	; (delete(+X,IGiven,IPure), var(X)) ->
	    I = [+X|IPure]
	; printf('Extending initial interpretation by +X%n',[]),
          I = [+_|IGiven]
	),
	I = [ITop|IRest],
	%% This is the root node:
	assert(node(ITop,pattern,[],[],0+0,0)),
	setval(level, 0),
	setval(sublevel(0),0),
	( \+ dp_flag(unit_extend,off) ->
%	  printf_verbose('Extending with unit clauses from input clause set...%n',[]),
	  unit_clauses_extend
	  ; true
        ),
	%% Finally, setup the branch with the initial interpretation:
	%% BUG:
	%% The initial interpretation together with the unit clauses 
	%% must be consistent - we will not detect unsatisfiable!!
	make_branch_unique(ITop,pattern,IRest).

print_leaf :- 
	getval(level,Level),
	branch_literal(Level,L,Kind),
	printf('[%Vw/%Vw/%Vw] ',[Level,L,Kind]).

    
branch_literals(Lits) :-
	branch_literals(0,Lits).

branch_literals(Level,[]) :-
	getval(level,BLevel),
	Level > BLevel.
branch_literals(Level,Res) :-
	findall(L,( branch_literal(Level+_SubLevel,L,_Kind), \+ L == true ),
	          LevelLits),
	NLevel is Level + 1,
	branch_literals(NLevel,RestLits),
	append(LevelLits,RestLits,Res).

