/*
 * This file contains the operations on the semantic tree
 */


%%%%%%%%%%%%%% satisfy - the main procedure %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% invariant: satisfy is called with a satisfiable branch
	
%% first, we try unit-resulting (UR) steps
satisfy :-
	\+ dp_flag(unit_extend,off),
	printf_verbose('Unit extension steps...%n',[]),
	% search from the root towards the leaf to find the next
	% universal literal that is required to be part of the UR-step
	setval(exists_falsified_clause,false),
	univ_extend_ctr_set_major(0),
	!,
	satisfy_UR.

satisfy :-
	% unit extend is off - directly jump to the next case
	satisfy_test_closure.

satisfy_UR :-
	%% UR-Steps
	( univ_extend_ctr_next(SLevel+SLMax) ->
	  %% So, the univ_extend_ctr counter is not yet exhauted
	  satisfy_UR_(SLevel+SLMax)
        ; %% exhausted the univ_extend_ctr counter -
	  %% have no more universal literal
	  satisfy_test_closure
        ).


satisfy_UR_(SLevel+SLMax) :-
	%% It is mandatory to use the node SLevel+SLMax
	%% This test can fail, when referrencing a pattern literal:
	setval(ur_step_applied,false),
	branch_literal(SLevel+SLMax,L,univ),
	%% Here we have found the next literal that is required to be part
	%% of the UR-step
	%% some optimisation to find the residue literal
	( dp_flag(unit_extend,positive) ->
	  Residue = +_
	; dp_flag(unit_extend,negative) ->
	  Residue = -_
	; Residue = _
        ),
	compl(L,LCompl),
	%% perform bulkwise all the UR-steps:
	findall(Residue/[(LCompl/(SLevel+SLMax))|CRestFalsification],
                (soi_clause(C),
	         ( (horn_clause_set, dp_flag(unit_extend,positive)) ->
		     delete(Residue,C,CHRest),
	             delete(LCompl,CHRest,CRest)
	           ; delete(LCompl,C,CComplRest),
	             delete(Residue,CComplRest,CRest)
	           ),
	           univ_falsified_clause(CRest,CRestFalsification)),
		  URStructure),
         !, satisfy_extend_units(URStructure).

satisfy_UR_(_SLevel+_SLMax) :-
	%% the universal literal did not exist, but the
	%% univ_extend_ctr counter is not yet exhausted
	univ_extend_ctr_inc,
	!, satisfy_UR.


%% satisfy_extend_units(LandRelAncs)
%% LandRelAncs is a list of pairs L/Falsifiction,
%% where L is a literal to be unit-extended with,
%% and Falsification is the list of immediate relevant ancestors
%% needed to derive L
%% Extend with all literals as given, modulo subsumption
satisfy_extend_units([]) :-
	%% In the horn clause case we do usually _only_ UR-steps
	%% throwing in the test for branch closure slightly more often
	%% seems to be advantageous
	horn_clause_set,
	getval(ur_step_applied,true),
	branch_closure(RelAncs),
	!,
	close_branch_and_satisfy(RelAncs).

satisfy_extend_units([]) :-
	%% try next round
	univ_extend_ctr_inc,
	!, satisfy_UR. 

satisfy_extend_units([(L/CRestFalsification)|RestURStructure]) :-
	%% Forward subsumption:
	\+ is_branch_literal_instance(L,_,_,univ),
	setval(exists_falsified_clause,true),
	term_depth_le_max(L),
%       the test is elsewhere:
%	term_depth_le_max(L),
	project_levels(CRestFalsification,RelAncs),
        unit_extend_univ(L,RelAncs,LLevel+LSubLevel),
	( unsatisfiable_branch(LLevel+LSubLevel,Anc+SubLevel) ->
	  branch_literals(Branch),
	  printf_verbose('Close branch %Dw after unit extension with %w%n',[Branch,L]), 
	  incval(branch_count),
	  !,
	  close_branch_and_satisfy([LLevel+LSubLevel,Anc+SubLevel])
	; 
	  setval(ur_step_applied,true),
	  !, satisfy_extend_units(RestURStructure)
        ).

satisfy_extend_units([(_L/_CRestFalsification)|RestURStructure]) :-
	%% L is subsumed or term depth is hit
	satisfy_extend_units(RestURStructure).


satisfy_test_closure :-
	branch_closure(RelAncs),
	!,
	close_branch_and_satisfy(RelAncs).
satisfy_test_closure :-
	satisfy_split.


%% satisfy_split
%% here we know that the branch cannot be closed.
%% Try to find a clause that is falsified and modify the current branch
%% by carrying out a split
satisfy_split :-
%% First try to find a clause that allows for a universal split:
        %% TRY:
	( \+ horn_clause_set ; dp_flag(unit_extend, off) ),
	setval(exists_falsified_clause,false),
	printf_verbose('Searching falsified univ splittable clause instance...%n',[]),
        soi_clause(C),
        falsified_clause_bounded(C,_UF,PF,[RL|RRest]),
%        falsified_clause(C,_UF,PF,[RL|RRest]),
%	setval(exists_falsified_clause,true),
	Residue = [RL|RRest],
	heuristics_c_univ([],Residue,PF,_RBestH,[L|_LRest]),
%	writeln(check(L)),
%	term_depth_le_max(L),
	\+ inconsistent_to_branch(L,_+_), %% loop check!!
	printf_verbose('Found splittable clause instance %Dw with residue %Dw%n',[C,Residue]),
	split(L,univ),
	make_branch_unique(L,univ,[]),
        !, 
	satisfy.

satisfy_split :-
	( \+ horn_clause_set ; dp_flag(unit_extend, off) ),
	printf_verbose('Searching falsified pattern splittable clause instance...%n',[]),
	soi_clause(C),
	falsified_clause_bounded(C,_UF,PF,[RL|RRest]),
	Residue = [RL|RRest],
	heuristics_c_pattern([],Residue,PF,_RBestH,[L|_LRest]),
%	(dp_flag(verbose,on) -> printf('%n',[]) ; true),
	printf_verbose('Found clause instance %Dw with residue %Dw%n',[C,Residue]),
	split(L,pattern),
	make_branch_unique(L,pattern,[]),
        !, 
	satisfy.

satisfy_split :-
	%% if there is no falsified clause (i.e. the term depth bound
	%% has not been hit) we can stop and report a model.
	getval(exists_falsified_clause,false), 
	setval(satisfiable,true), !.

satisfy_split :-
% here we know that there is a falsifiable input clause 
% but we hit the term depth limit
% Thus, the term depth limit has to be increased
	increase_term_depth_limit,
	!,
        satisfy.


increase_term_depth_limit :- 
	get_term_depth_max(M),
	dp_flag(term_depth,(_Init,Inc)),
	M1 is M+Inc,
	set_term_depth_max(M1), 
	printf_verbose('--------------------------------%n',[]),
	printf_verbose('INCREASE TERM DEPTH LIMIT TO %w%n',[M1]),
	printf_verbose('--------------------------------%n',[]),
	%% unfortunately, have to clear information about 
	%% current state of computation for units, as with new depth bound, 
	%% new units may well be derivable.
	( \+ dp_flag(unit_extend,off) ->
	  univ_extend_ctr_clear
        ; true
        ).

%% unit_clauses_extend
%% extend with the unit clauses from the input clause set
unit_clauses_extend :-
	soi_clause([L]),
	unit_extend_univ(L,[],_+_),
	fail.
unit_clauses_extend.

%% A handy abbreviation for often used components of node
branch_literal(Level+SubLevel,L,Kind) :-
	node(L,Kind,_,_,Level+SubLevel,_LG).
ground_branch_literal(Level+SubLevel,LG,Kind) :-
	node(_L,Kind,_,_,Level+SubLevel,LG).

%% split(L,Kind)
%% The elementary split operation
%% L is the literal to split with,
%% Kind is either 'pattern' or 'univ'
split(L,Kind) :-
	getval(level,Level),
	NLevel is Level + 1,
	compl(L,LNeg),
	%% BUG FIX in version 0.9.1: this has to be skolemised
	( Kind == univ ->
	  skolemize(LNeg,LNegFinal)
	; LNegFinal = LNeg
        ),
	%% Start the new sublevel counting
	SubLevel is 0,
	setval(sublevel(NLevel),SubLevel),
	univ_extend_ctr_set_minor(NLevel,SubLevel),
	ground_variables(L,LG),
% Playground:	
%	( Kind == univ ->
%	  asserta(node(L,Kind,[LNeg],[],NLevel+SubLevel,LG))
%        ; assert(node(L,Kind,[LNeg],[],NLevel+SubLevel,LG))
%        ),
	assert(node(L,Kind,[LNegFinal],[],NLevel+SubLevel,LG)),
	printf_verbose('Split (type \'%w\') with %w%n',[Kind,L]),
	setval(level,NLevel),
	incval(split_count).

%% unit_extend_univ(L,RelAncs,LInsertLevel+LSubLevel)
%% The 'one-armed' split - is only sound for universal literals,
%% Extend with literal L, return in LInsertLevel+LSubLevel the level
%% of the new created node
%% RelAncs are the immediately relevant ancestor that justify L
unit_extend_univ(L,RelAncs,LInsertLevel+LSubLevel) :-
% Playground:
% Insert as rootmost in the tree as possible:
	max_level([(0+0)|RelAncs],LInsertLevel+LInsertSubLevel),
% ...or insert at current level:
%	getval(level,LInsertLevel),
%	getval(sublevel(LInsertLevel),LInsertSubLevel),
%       currently disabled:
	back_subsume(L,LInsertLevel+LInsertSubLevel),
	incval(sublevel(LInsertLevel)),
	getval(sublevel(LInsertLevel),LSubLevel),
	asserta(node(L,univ,[],RelAncs,LInsertLevel+LSubLevel,L)),
	printf_verbose('Unit extension with %Vw to level %w%n',[L,LInsertLevel]),
	incval(unit_count), !.

%% back_subsume(L,Level+SubLevel)
%% See if literal L, which is the label of node at level Level+SubLevel
%% back subsumes literal K. 
%% Take care of dependencies!
back_subsume(L,Level+SubLevel) :-
	dp_flag(back_subsumption,on),
	node(BL,univ,BSibling,BRelAnc,BLevel+BSubLevel,BLG),
	%% we may allow L to subsume BL if L is strictly more towards
	%% the root than BL, because then L can be thought of as inserted
	%% before BL, so that BL would be `forward' subsumed.
	strictly_preceedes_anc(Level+SubLevel,BLevel+BSubLevel),
	more_general(L,BL),
	printf_verbose('Delete node %w by back subsumption with %w%n',[BL,L]),
        incval(back_subsume_count),
	retract(node(BL,univ,BSibling,BRelAnc,BLevel+BSubLevel,BLG)),
	asserta(node(true,univ,[],BRelAnc,BLevel+BSubLevel,true)),
	back_subsume(L,Level).
%% Playground - could also back subsume splits
back_subsume(_L,_Level+_SubLevel).


	
%% shift_branch(Level,RelAncs): 
%% The current branch with node at level Level is considered as closed, 
%% therefore find next open branch.
%% This involves making the interpretation representation of the candidate
%% branch unique.
%% fail if no next such open branch exists
shift_branch(0,_RelAncs) :-
	retract_all(node(_L,_,_,_,0+_,_)),
	setval(refuted,true),
	!, fail.
shift_branch(Level,RelAncs) :-
	%% first get the touched nodes of this Level
	relanc_level(Level,RelAncs,LevelRelAncs),
	shift_branch_(Level,LevelRelAncs).

shift_branch_(Level,LevelRelAncs) :-
	node(_,Kind,[Sibling],_,Level+SubLevel,_LG),
	%% so, there is a sibling
	( memberq(Level+SubLevel,LevelRelAncs) ; dp_flag(level_cut,off) ),
	% so, have to go the the next branch
	restrict_to_level(Level,LevelRelAncs,XLevelRelAncs),
	retract_all(node(_,_,_,_,Level+_,_)),
	NewSubLevel = 0,
	setval(sublevel(Level),NewSubLevel),
	univ_extend_ctr_set_minor(Level,NewSubLevel),
	ground_variables(Sibling,SiblingG),
	( Kind == univ ->
	  asserta(node(Sibling,Kind,[],XLevelRelAncs,Level+NewSubLevel,SiblingG))
	; assert(node(Sibling,Kind,[],XLevelRelAncs,Level+NewSubLevel,SiblingG))
        ),
	%% now see if the new branch is consistent:
	( unsatisfiable_branch(LAnc+LSubLevel,LCAnc+LCSubLevel) ->
	  branch_literals(Branch),
	  branch_literal(LAnc+LSubLevel,HL,_),
	  branch_literal(LCAnc+LCSubLevel,HLC,_),
	  printf_verbose('Close branch %w by inconsistency between %w - %w%n',
                         [Branch,HL,HLC]), 
	  incval(branch_count),
	  !,
	  shift_branch(Level,[LAnc+LSubLevel,LCAnc+LCSubLevel])
        ; make_branch_unique(Sibling,Kind,[]), 
	  !
        ).

shift_branch_(Level,LevelRelAncs) :-
	( node(_,_Kind,[_Sibling],_,Level+_SubLevel,_LG) -> 
	  printf_verbose("Cutting split at level %w\n",
                            [Level]), 
          incval(level_cut_count)
        ; true
        ),
	retract_all(node(_,_,_,_,Level+_,_)),
	restrict_to_level(Level,LevelRelAncs,XLevelRelAncs),
	decval(level),
	BackLevel is Level-1, !,
	shift_branch(BackLevel,XLevelRelAncs).

%% unsatisfiable_branch(LAnc+LSubLevel,LCAnc+LCSubLevel)
%% succeeds if there is an unresolvable contradiction between
%% the two nodes
unsatisfiable_branch(LAnc+LSubLevel,LCAnc+LCSubLevel) :-
	branch_literal(LAnc+LSubLevel,L,univ),
	inconsistent_to_branch(L,LCAnc+LCSubLevel).

%% the universally quantified literal L is inconsistent to the current branch
inconsistent_to_branch(L,LCAnc+LCSubLevel) :-
        compl(L,LC),
	( branch_literal(LCAnc+LCSubLevel,LC,univ) 
	; ( branch_literal(LCAnc+LCSubLevel,K,pattern) , more_general(LC,K) )
	).

%% restrict_to_level(Level,LevelRelAncs,XLevelRelAncs)
%% restrict to the Level+SubLevel pairs in LevelRelAncs, where Level
%% is given in the first argument
restrict_to_level(_Level,[],[]).
restrict_to_level(Level,[(Level+_SubLevel)|RestRelAncs],
	            ResRestRelAncs) :- 
	restrict_to_level(Level,RestRelAncs,ResRestRelAncs), !.
restrict_to_level(Level,[(HLevel+HSubLevel)|RestRelAncs],
	           [(HLevel+HSubLevel)|ResRestRelAncs]) :- 
	restrict_to_level(Level,RestRelAncs,ResRestRelAncs).

%% relanc_level(Level,RelAncs,Res)
%% Extend the set RelAncs of relevant ancestors by the immediately
%% relevant ancestors of Level
relanc_level(Level,RelAncs,Res) :-
	relanc_level(Level,RelAncs,[],Res).

relanc_level(_Level,[],SoFar,SoFar).
relanc_level(Level,[(Level+SubLevel)|Rest],SoFar,Res) :-
	node(_L,_Kind,_Sibling,RelAncs,Level+SubLevel,_LG),
	append(RelAncs,Rest,NewRelAncs),
	( memberq((Level+SubLevel),SoFar) ->
	    NewSoFar = SoFar
	  ; NewSoFar = [(Level+SubLevel)|SoFar]
        ),
	relanc_level(Level,NewRelAncs,NewSoFar,Res).
relanc_level(Level,[(FLevel+FSubLevel)|Rest],SoFar,Res) :-
	FLevel < Level,
	( memberq((FLevel+FSubLevel),SoFar) ->
	    NewSoFar = SoFar
	  ; NewSoFar = [(FLevel+FSubLevel)|SoFar]
        ),
	relanc_level(Level,Rest,NewSoFar,Res).
relanc_level(Level,[(FLevel+FSubLevel)|Rest],SoFar,Res) :-
	%% FLevel > Level
	node(_L,_Kind,_Sibling,FRelAncs,FLevel+FSubLevel,_LG),
	append(FRelAncs,Rest,NewRelAncs),
	relanc_level(Level,NewRelAncs,SoFar,Res).

% make_branch_unique(L,Kind,Rest)
% Extend the current branch whose leaf is L and has kind Kind
% with splits over all the literals in Rest
% still have to check between all conflicts between branch
% up to (not including) L and all literals in [L|Rest]
% guaranteed that no literal from [L|Rest] is contained as a variant
% on branch up to (not including) L
%
make_branch_unique(_L,univ,_Rest).
make_branch_unique(L,pattern,Rest) :-
	copy_term(L,LC),
	compl(LC,LCCompl),
	branch_literal(_Level+_SubLevel,LCCompl,_Kind),
	\+ member_variant(LC,Rest),
	\+ member_variant(LCCompl,Rest),
	\+ branch_literal_variant(_,LC,pattern),
	\+ branch_literal_variant(_,LCCompl,pattern),
	\+ is_branch_literal_instance(LC,_,_,univ),
	\+ is_branch_literal_instance(LCCompl,_,_,univ),
	make_branch_unique(L,pattern,[LCCompl|Rest]).
% here we know that all conflicts of first literal
% with rest of branch have been collected in Rest
make_branch_unique(_L,pattern,[]).
        % nothiing to do because assume that L is as a leaf on branch
make_branch_unique(_L,pattern,[K|Rest]) :-
        printf_verbose('Uniquify branch: ',[]),
	incval(uniquify_count),
	(ground(K) -> 
	    Kind = univ ; 
	    Kind = pattern),
	split(K,Kind),
	make_branch_unique(K,pattern,Rest).
	

%% falsified_immediately_literal((Level+SubLevel),L,Kind)
%% Instatiate the literal L so that it is falsified by the node
%% at level Level+SubLevel of kind Kind

% look for universally falsified literals first
falsified_immediately_literal((Level+SubLevel),L,univ) :-
	%% universal case
	compl(L,LNeg),
%	copy_term(LNeg,LNegCopy),
	branch_literal(Level+SubLevel,LNeg,univ).
%	done by caller:
%	(variant(LNeg,LNegCopy) -> ! ; true).

falsified_immediately_literal(Level+SubLevel,L,pattern) :-
	%% pattern case
	compl(L,LNeg),
	branch_literal(Level+SubLevel,LNeg,pattern),
%	preceedes_anc(Level+SubLevel,MaxLevel+MaxSubLevel),
	%% check that we have a literal that will not also be falsified
	%% by a universal literal - make this case not ovelap with 
	%% the previous case
	\+ is_branch_literal_instance(LNeg,_,_,univ),
	%% check LNeg has not become immediately true
	%% by universal literal
	\+ is_branch_literal_instance(L,_,_,univ),
	branch_literal(Level+SubLevel,LNegProducer,pattern),
	%% see if we have the most specific branch literal
	%% that produces the instance LNeg
	( ground(LNegProducer) -> true
        ; \+ ( branch_literal(_+_,LNegTest,pattern),
	       strictly_more_general(LNegProducer,LNegTest),
	       more_general(LNegTest,LNeg)
	     ),
	   %% check LNeg has not become immediately true
	   %% by pattern  literal
           \+ ( branch_literal(_+_,K,pattern),
	        compl(K,KNeg),
	        strictly_more_general(LNegProducer,KNeg),
	        more_general(K,L)
	      )
	).

%% a special case of the previous procedure
%% used to check that the result of calling the previous procedure
%% is still valid. i.e. the instantiated L is still falsified by the same
%% interval 
check_falsified_immediately_literal(Level+SubLevel,L,pattern) :-
	compl(L,LNeg),
	\+ is_branch_literal_instance(LNeg,_,_,univ),
	\+ is_branch_literal_instance(L,_,_,univ),
	branch_literal(Level+SubLevel,LNegProducer,pattern),
	( ground(LNegProducer) -> true
        ;  %% check LNeg has not become immediately true
	   %% by pattern  literal
           \+ ( branch_literal(_+_,K,pattern),
	        compl(K,KNeg),
	        strictly_more_general(LNegProducer,KNeg),
	        more_general(K,L)
	      )
	).


%% branch_literal_variant(Level+SubLevel,L,Kind).
%% The literal label of node Level+SubLevel of kind Kind is a varaint of L
branch_literal_variant(Level+SubLevel,L,Kind) :- 
        branch_literal(Level+SubLevel,K,Kind),
	variant(K,L), !. % gree cut - one is enough

% is_branch_literal_instance(L,Level,K,Kind).
% the literal L is an instance of literal K, which is at level Level 
% and has kind Kind
is_branch_literal_instance(L,Level+SubLevel,K,Kind) :-
	branch_literal(Level+SubLevel,K,Kind),
	more_general(K,L), !. %% strictly_more_general ???

%% falsified_clause_bounded(Clause,UF,PF,Residue)
%% Compute an instance of clause Clause that is falsified by the
%% current branch.
%% Thereby Clause is split into three parts. One part consists of those
%% literals that are falsified by universal branch literals. The levels
%% of those are collected in UF ('Universal Falsification')
%% The secon part consists of literals that are falsified by pattern
%% literals by variantship. These literals cannot be 'repaired' by a split.
%% SImilar to UF, the falsifying nodes are collected in PF
%% The third part are those that are falsified by a pettern literal, and
%% that can be satisfied with a split. This third part is the Residue.
%% For the Residue literals we check that they are within the current
%% depth bound
falsified_clause_bounded(Clause,UF,PF,Residue) :-
	falsified_clause(Clause,UF,PF,Residue),
	%% this is used below in main loop that there is a falsified
	%% clause at all, so that it is worth increasing the depth
	%% bound
	setval(exists_falsified_clause,true),
	term_depth_le_max(Residue).

%% See description of the previous procedure.
falsified_clause([],[],[],[]).
falsified_clause([L|RClause],[L/(Level+SubLevel)|RestUF],PF,Residue) :-
	% case 1: L is falsified by a universal literal:
%        copy_term(RClause,RClauseC),
        copy_term(L,LC),
	falsified_immediately_literal(Level+SubLevel,L,univ),
	(variant(L,LC) -> ! ; true),
%	( variant(RClause,RClauseC) -> 
%	  writeln(delete(L,RClause,RClauseC)), ! 
%        ; true), 
	falsified_clause(RClause,RestUF,PF,Residue).

falsified_clause([L|RClause],UF,[L/(Level+SubLevel)|RestPF],Residue) :-
	%% case 2: pattern case, and cannot update current interpretation
	compl(L,LNeg),
	branch_literal(Level+SubLevel,LNeg,pattern),
        branch_literal(Level+SubLevel,KNeg,pattern),
	variant(LNeg,KNeg),
	falsified_clause(RClause,UF,RestPF,Residue),
	variant(LNeg,KNeg).

falsified_clause([L|RClause],UF,PF,[L|RestResidue]) :-
	%% case 3: pattern case, but can update current interpretation
	falsified_immediately_literal(Level+SubLevel,L,pattern),
	falsified_clause(RClause,UF,PF,RestResidue),
	% test if L is falsified in the same interval:
	check_falsified_immediately_literal(Level+SubLevel,L,pattern),
	compl(L,LC),
	\+ branch_literal_variant(_+_,LC,pattern).
	%% this guarantees that the literal L is splittable
	%% (together with the fact that falsified_immeditely_literal
	%% never holds for a literal L that is an instance of a universal
	%% literal)


%% a special case of falsified_clause,
%% but allow only universal literals
%% used to perform unit-resulting steps
univ_falsified_clause([],[]).
univ_falsified_clause([L|RClause],[L/(Level+SubLevel)|RestRes]) :-
        copy_term(L,LC),
	compl(L,LNeg),
	branch_literal(Level+SubLevel,LNeg,univ),
	univ_extend_ctr_get_minor(Level,CheckSubLevel),
	SubLevel =< CheckSubLevel,
	% most general case:
	(variant(L,LC) -> ! ;  true), 
	univ_falsified_clause(RClause,RestRes).

%% another special case
%% 
ground_falsified_clause([],[],[]).
ground_falsified_clause([L|RClause],[L/(Level+SubLevel)|RestUF],PF) :-
	% case 1: L is falsified by a universal literal:
        copy_term(L,LC),
	falsified_immediately_literal(Level+SubLevel,L,univ),
	(variant(L,LC) -> ! ;  true), 
	ground_falsified_clause(RClause,RestUF,PF).

ground_falsified_clause([L|RClause],UF,[L/(Level+SubLevel)|RestPF]) :-
	%% case 2: pattern case, and cannot update current interpretation
        copy_term(L,LC),
	compl(L,LNeg),
	ground_branch_literal(Level+SubLevel,LNeg,pattern),
	(variant(L,LC) -> ! ;  true), 
	ground_falsified_clause(RClause,UF,RestPF).

%% test if the current branch is closed
branch_closure(RelAncs) :-
	%% special case for Horn clause set, if flag setting permits
	horn_clause_set, 
	\+ dp_flag(unit_extend, off),
	!,
	printf_verbose('Testing branch closure...%n',[]),
	soi_clause(C),
	%% TRY
	%% \+ member(+_,C), %% search for all-negative clause
%	falsified_clause(C,UF,PF,[]),
	ground_falsified_clause(C,UF,PF),
	project_levels(UF,UFRelAncs), 
	project_levels(PF,PFRelAncs), 
	append(UFRelAncs,PFRelAncs,RelAncs),
	branch_literals(Branch),
	printf_verbose('Close branch %Dw%n%twith clause%Dw%n',
             [Branch,C]), 
	incval(branch_count).

branch_closure(RelAncs) :-
	printf_verbose('Testing branch closure...%n',[]),
	soi_clause(C),
%	falsified_clause(C,UF,PF,[]),
	ground_falsified_clause(C,UF,PF),
	project_levels(UF,UFRelAncs), 
	project_levels(PF,PFRelAncs), 
	append(UFRelAncs,PFRelAncs,RelAncs),
	branch_literals(Branch),
	printf_verbose('Close branch %Dw%n%twith clause%Dw%n',
             [Branch,C]), 
	incval(branch_count).


close_branch_and_satisfy(RelAncs) :-
	getval(level,Level),
	!,
	shift_branch(Level,RelAncs),
	!, satisfy.


%% heuristics_c_univ(ResidueHead,[L|RResidue],PF,BestH,Best)
%% Assume that a Residue has been computed and that it split
%% in three parts ResidueHead - L - RResidue, and the Relevant ancestors 
%% are PF. Try to find a literal in the Residue that can be universally
%% split. Fails if no such literal can be found
heuristics_c_univ(ResidueHead,[L|RResidue],PF,BestH,Best) :-
	( 
	  ( dp_flag(univ_split,on) -> 
	    \+ ground(L), PF == [], disjoint(L,ResidueHead), disjoint(L,RResidue) )
        ; 
	  ground(L) 
        ),
	!,
	%% Yes, here we know that L can be universally split
	%% select one with least complexity
	complexity(L,H1), H is 1000 - H1,
	( heuristics_c_univ([L|ResidueHead],RResidue,PF,RBestH,RBest) ->
	  ( H > RBestH ->  
	    BestH = H,
	    Best = [L|RResidue]
	  ; BestH = RBestH,
	    RBest = [K|RR],  
	    Best = [K,L|RR]
	  )
        ; BestH = H,
	  Best = [L|RResidue]
	).
heuristics_c_univ(ResidueHead,[L|RResidue],PF,BestH,Best) :-
	heuristics_c_univ([L|ResidueHead],RResidue,PF,BestH,Best).

%% find a literal to be split by pattern.
%% here we select one of least complexity
heuristics_c_pattern(_ResidueHead,[],_PF,-1,_Best).
heuristics_c_pattern(ResidueHead,[L|RResidue],PF,BestH,Best) :-
	heuristics_c_pattern([L|ResidueHead],RResidue,PF,RBestH,RBest),
	complexity(L,H1), H is 1000 - H1,
%	H is 0,
	( H > RBestH ->
	  BestH = H,
	  Best = [L|RResidue]
	; BestH = RBestH,
%	  ( RBest == [] -> printf('IMPOSSIBLE') ; true ),
	  RBest = [K|RR],  
	  % RBest == [] impossible because then 'then' case applies
	  Best = [K,L|RR]
	).

%% Select components from a falsification list
project_levels([],[]).
project_levels([_/(Level+SubLevel)|R],Res) :-
	project_levels(R,RRes),
	add_element((Level+SubLevel),RRes,Res), !.
project_literals([],[]).
project_literals([L/(_+_)|R],[L|RRes]) :-
	project_literals(R,RRes).

%% The term depth of term T does not exceed the current limit
term_depth_le_max(T) :-
	get_term_depth_max(MaxTermDepth),
	term_depth_le(T,MaxTermDepth), !.


set_term_depth_max(M) :-
%	statistics(times, [Time, _, _]),
%	setval(term_depth_inc_time,Time),
	setval(term_depth_max,M).

get_term_depth_max(M) :-
	getval(term_depth_max,M).

%% computes a maximal element of list of levels:
max_level([(Level+SubLevel)|Rest],ResLevel+ResSubLevel) :- 
	max_level_(Rest,Level+SubLevel,ResLevel+ResSubLevel), !.
max_level_([],SoFarLevel+SoFarSubLevel,SoFarLevel+SoFarSubLevel).
max_level_([(Level+SubLevel)|Rest],SoFarLevel+SoFarSubLevel,ResLevel+ResSubLevel) :- 
	( \+ strictly_preceedes_anc(Level+SubLevel,SoFarLevel+SoFarSubLevel) ->
	  max_level_(Rest,Level+SubLevel,ResLevel+ResSubLevel)
        ; max_level_(Rest,SoFarLevel+SoFarSubLevel,ResLevel+ResSubLevel)
        ).

%% Lexicographical comparison of levels:
strictly_preceedes_anc(Level1+SubLevel1,Level2+SubLevel2) :- 
	( Level1 < Level2 ->
	  true
        ; ( Level1 = Level2, SubLevel1 < SubLevel2 ) ->
	  true
        ; false
        ).

preceedes_anc(Level1+SubLevel1,Level2+SubLevel2) :- 
	( Level1 < Level2 ->
	  true
        ; ( Level1 = Level2, SubLevel1 =< SubLevel2 ) ->
	  true
        ; false
        ).

