%%%%%%%%%%%%%%%% MISC
%
% Various helper procedures
%
printf_verbose(Format,Args) :-
	dp_flag(verbose,on), !,
	getval(level,Level),
	printf('[%w] ',[Level]),
        printf(Format,Args),
	flush(user).
printf_verbose(_Format,_Args).

more_general(K,L) :-
	% assume that K and L are variable disjoint:
	compare_instances(R,K,L),
	(R == '>' ; R == '=').

strictly_more_general(K,L) :-
	% assume that K and L are variable disjoint:
	compare_instances('>',K,L).


compl(- Literal, + Literal) :- !.
compl(+ Literal, - Literal).

compl_literals([],[]).
compl_literals([L|R],[LCompl|RCompl]) :-
	compl(L,LCompl),
	compl_literals(R,RCompl).


%% membership-test - based on variants
member_variant(L,[K|Rest]) :-
	( variant(L,K) ->
	  true
	; member_variant(L,Rest)
        ).

%% The term X is within the depth bound Max:
term_depth_le(X,Max) :- 
	var(X),  !, Max >= 0.
term_depth_le(X,Max) :- 
	atomic(X), !, Max >= 0.
%term_depth_le(X,Max) :- 
%	Max > 0,
%	functor(X,':',2), %% sorts - ignore sort term
%	NMax is Max-1, !,
%	term_depth_le_(X,1,1,NMax).
term_depth_le(X,Max) :- 
	Max > 0,
	functor(X,_F,Arity),
%	Max >= Arity,
	NMax is Max-1, 
%	NMax is Max-Arity, 
	term_depth_le_(X,1,Arity,NMax).
term_depth_le([],_Max) :- !.
term_depth_le([F|R],Max) :- !, 
	term_depth_le(F,Max), term_depth_le(R,Max).
term_depth_le_(_X,I,MaxI,_Max) :- I > MaxI, !.
term_depth_le_(X,I,MaxI,Max) :-
	arg(I,X,Arg),
	term_depth_le(Arg,Max),
	I1 is I+1,
	term_depth_le_(X,I1,MaxI,Max).


%% complexity of terms

complexity(X,0) :- 
	var(X), !.
complexity(sk(_X),0) :- 
	!.
complexity(Term : _Type,C) :- 
	!, complexity(Term,C).
complexity(X,1) :- 
	atomic(X), !.
%complexity(X,SoFar,Res) :- 
%	functor(X,':',2), %% disregard sort terms
%	!, 
%	complexity_(X,1,1,SoFar,Res).
complexity(X,Res) :- 
	functor(X,_F,Arity),
%	HSoFar is SoFar+Arity,
	complexity_(X,1,Arity,0,HRes),
%	Res is HRes+Arity.
	Res is HRes+1.
complexity_(_X,I,MaxI,SoFar,SoFar) :- I > MaxI, !.
complexity_(X,I,MaxI,SoFar,Res) :-
	arg(I,X,Arg),
	complexity(Arg,ArgRes),
	I1 is I+1,
	max(ArgRes,SoFar,NewMax),
	complexity_(X,I1,MaxI,NewMax,Res), !.

ground(T) :-
	\+ nonground(T).
	
disjoint(T1,T2) :-
	term_variables(T1,T1Vars),
	term_variables(T2,T2Vars),
	intersectq(T1Vars,T2Vars,[]).


%% convert between prolog lists and lists in parenthesis notation
pl_list( (K, L), [K | PL_L] ) :- pl_list(L, PL_L), !.
pl_list( (K; L), [K | PL_L] ) :- pl_list(L, PL_L), !.
pl_list( K, [K] ) :- K \== true.
pl_list( true, [] ).

negate(- Literal, Literal) :- !.
negate(Literal, - Literal).

negate_list([],[]).
negate_list([H|T],[NH|NT]) :-
	negate(H,NH),
	negate_list(T,NT).

negative_clause([]).
negative_clause([-_A|R]) :-
	negative_clause(R).

max_list([N],N) :- !.
max_list([N|R],Max) :- max_list(R,RMax), max(N,RMax,Max).

add_element(Elem, Set, Set) :-
        memberchk(Elem, Set),
        !.
add_element(Elem, Set, [Elem|Set]).


norm_clause([],[]).
norm_clause([false|L],LR) :- !, norm_clause(L,LR).
norm_clause([-true|L],LR) :- !, norm_clause(L,LR).
norm_clause([K|L],[K|LR]) :- norm_clause(L,LR), !.

norm_list([],[]).
norm_list([~K|L],[-K|LR]) :- !, norm_list(L,LR).
norm_list([K|L],[K|LR]) :- norm_list(L,LR), !.

is_pos(- _) :- !, fail.
is_pos(_).
is_neg(- _).


litvarsets([],[]).
litvarsets([L|R],[VL|VR]) :-
	term_variables(L,VL),
	litvarsets(R,VR).

commonvars([],[]).
commonvars([F|R],L) :-
	unionqlists(R,UR),
	intersectq(F,UR,PosLits),
	commonvars(R,R1),
	unionq(PosLits,R1,L).



nonmemberq(HArg,[Arg|Tail]) :-
        %HArg ~= Arg, !, nonmemberq(HArg,Tail).
        HArg \== Arg, !, nonmemberq(HArg,Tail).
nonmemberq(_,[]).

memberq(HArg,[Arg|_]) :-
        HArg == Arg, !.
memberq(Arg,[_|Tail]) :-
        memberq(Arg,Tail).

intersectq([], _, []).

intersectq([Element|Residue], Set, Result) :-
        memberq(Element, Set),
        !,                    
        Result = [Element|Intersection],
        intersectq(Residue, Set, Intersection).

intersectq([_|Rest], Set, Intersection) :-
        intersectq(Rest, Set, Intersection).

unionqlists([],[]).
unionqlists([F|R],Res) :-
	unionqlists(R,HRes),
	unionq(F,HRes,Res).

unionq([], Set2, Set2).
unionq([Element|Residue], Set, Union) :-
       memberq(Element, Set), !,
       unionq(Residue, Set, Union), !.
unionq([Element|Residue], Set, [Element|Union]) :-
       unionq(Residue, Set, Union).

ground_variables(Term,TermG) :-
	copy_term(Term,TermG),
	term_variables(TermG,Vars),
	groundify(Vars).

groundify([]).
groundify([V|R]) :-
	V = '$',
	groundify(R).

:- setval(skolemctr,0).

skolemize_varlist([]).
skolemize_varlist([sk(Nr)| MoreVars]) :-
	getval(skolemctr,Nr),
	incval(skolemctr),
	skolemize_varlist(MoreVars).

skolemize(T,TC) :-
	copy_term(T,TC),
	term_variables(TC,TCV),
	skolemize_varlist(TCV).

	
